/**
 * \file error.h
 * Used to work with user's exceptions.
 * It creates stack of error messages that can be caught and outputed to the screen.
 * Initial error is generated somewhere, and passed to called function, where mode description can be added. That is stack.
 
 * \todo Remove from THE CODE error class, output class, move enclosed classes out, remove variabilities in function callings, change all enum's to strings. In that case, probably, the code can be understood.
 *
 * \author Developed under supervision of the PI Yuri Shprits
 */


#ifndef ERROR_MESSAGES_H
#define ERROR_MESSAGES_H

#include <string>
#include <vector>
#include <stdarg.h>
#include <stdio.h>

using namespace std;

/**
 * \brief Hold some information about an error in the code.
 */
class single_error {
public:
	/// Error code.
	string code;

	/// Error message.
	string msg;

	/// Constructor.
	single_error(string code) {
		// store error code and empty message
		this->code = code;
		this->msg = "";
	}

	/// Constructor.
	single_error(string code, string msg) {
		// store error code and message
		this->code = code;
		this->msg = msg;
	}
};

//typedef vector<error_msg> error_stack;

/**
 * \brief Error message - stack of single_error%s.
 */
class error_msg {
public:
	/// Stack of errors.
	vector<single_error> errors_stack;

	/// Constructor.
	error_msg();
	/** Constructor.
	 * one message added to the stack.
	 */
	error_msg(char * code) {
		errors_stack.push_back(single_error(code));
	}
	//error_msg(string code, string msg) {
	//	errors_stack.push_back(single_error(code, msg));
	//}

	/** Constructor.
	 * one message added to the stack.
	 */
// !!!	error_msg(char * code, char * msg, ...) {
		error_msg(const char * code, const char * msg, ...) {

		// converting msg and args into one message and add error in stack with this message

		int len;
		va_list args;
		va_start( args, msg );
		char *buffer = new char[1024];

//!!!		len = _vscprintf( msg, args ) + 1;
		len = vsnprintf(buffer, 1024, msg, args ) + 1;
		if (len == 0) throw error_msg("OUTPUT", "Output does not work :-(");

		errors_stack.push_back(single_error(code, buffer));

		delete buffer;
	}

	/**
	 * Add one error message to the stack.
	 */
	void add(single_error err) {
		errors_stack.push_back(err);
	}

	/**
	 * Add one error message to the stack.
	 */
	void add(string code) {
		errors_stack.push_back(single_error(code));
	}

	/**
	 * Add one error message to the stack.
	 */
	void add(string code, string msg) {
		errors_stack.push_back(single_error(code, msg));
	}

	/**
	 * Return all messages as a text.
	 */
	string what() {
		unsigned i;
		string result = "";
		for (i = 0; i < errors_stack.size(); i++) {
			// return all stack of errors as a string
			result = result + errors_stack[i].code + ": " + errors_stack[i].msg + "\n";
		}
		return result;
	}

};

#endif
