/**
 * \file AdditionalSourcesAndLosses.cpp
 * Sources and losses calculation
 *
 * \author Developed under supervision of the PI Yuri Shprits
 */

#include <math.h>
#include "AdditionalSourcesAndLosses.h"

#define err 1e-3

/** Loading (or just calculate) sources and losses
 *
 * \param &SL - %PSD parameters
 * \param grid - grid
 * \param numberOfIterations - number of time steps total
 * \param timeStep - time step size
 */
void AdditionalSourcesAndLosses::Create_SL(Parameters_structure::SL_structure &SL, Grid &grid, int numberOfIterations, double timeStep) {


	/// todo: Somewhere add check to Local Diffusion... ???

	// Check do we have a magnetopause
	if (SL.useLmp == true & SL.Lmp.size_x > 0 & SL.Lmp.size_y > 0)
	{
		// magnetopause determined
		//grid.alpha.size;
		//grid.L.size;
		//grid.pc.size;

		this->timeStep = timeStep;

		// Create our losses array, after we will fill it
		arr.AllocateMemory(grid.L.size, grid.pc.size, grid.alpha.size);
	}
		

	// \todo: add code here

	// Add magnetopause shadowing here
	/*
	1. Load data from file (I think whole file)
	2. File should contain information about L* magnitomause position for current pitch angle
	3. Check grid and number of interations
	4. Maybe do linear interpolation for interation  (maybe later)

	5. Rest in update!
	*/
}

/**
 * Update additional sources and losses for parameters
 * \param iteration - of SL
 * \param &SL - sources and losses parameters structure
 * \param &grid - grid to update
 */
void AdditionalSourcesAndLosses::Update(int iteration, Parameters_structure::SL_structure &SL, Grid &grid){
	// for each iteration build SL array corresponded to losses by magnetopause

	if (SL.useLmp == true) 
		for (int il = 0; il < grid.L.size; il++)
			for (int im = 0; im < grid.pc.size; im++)
				for (int ia = 0; ia < grid.alpha.size; ia++)
				{
					// We are in the right position of the array

					if (grid.L.arr[il][im][ia] >= SL.Lmp[iteration][ia]) // our client
					{
                        ///drift time
						double tau_d;
						tau_d = AdditionalSourcesAndLosses::drift_time(grid.pc.arr[il][im][ia], grid.alpha.arr[il][im][ia], grid.L.arr[il][im][ia]);
						arr[il][im][ia] = exp(-1 * this->timeStep / tau_d);
					}

					
					else
						arr[il][im][ia] = 1;
				}
			
	/*
	1. Take loaded file and apply interation to that file
	2. Build 3d array of current magnetopause shadowing
	*/

}

// Walt (4.47)
/**
 * Multiplicator due to losses into magnetopause
 * 
 * \param pc - pc value
 * \param alpha - alpha value
 * \param L - L value
 */
double AdditionalSourcesAndLosses::drift_time(double pc, double alpha, double L)
{
	double Cd = 1.557E4 / 86400; // days for electrons
	double gamma = sqrt( 1 + pc*pc / (VC::mc2 * VC::mc2) );
	
	double td = Cd* (1 / L ) * gamma / (gamma * gamma - 1) * (1 - 1/3 * pow(sin(alpha),0.62) );

	return td;
}