/**
 * \file "BoundaryConditions.h"
 * Boundary condition class.
 *
 * Is a child of class Grid and holds 2D array of boundary values for function or function derivative.
 * Additional third dimension is time.
 *
 * \author Developed under supervision of the PI Yuri Shprits
 */

#ifndef BoundaryConditions_H
#define BoundaryConditions_H

#include <string>
#include "../Matrix/Matrix.h"
#include "../Parameters/Parameters.h"
#include "../VariousFunctions/variousFunctions.h"

using namespace std;

/**
 * \brief Holds upper and lower boundary conditions
 *
 * Allows for loading, creating, and updating boundary conditions.
 */
class BoundaryCondition {
public:
    /// 2D matrix of boundary conditions
	Matrix2D<double> arr;

	/// Boundary condition parameters
	Parameters_structure::BoundaryCondition BC_parameters;

	/// Type: constant function/constant derivative
	string	calculationType;
	/// Initial type: how to set up boundary conditions. (Like read from a file, etc)
	string	type;

	// functions
	void MakeBoundaryCondition(Parameters_structure::BoundaryCondition parameters, Matrix2D<double> psd2DSlice, Matrix2D<double> gridElement1, Matrix2D<double> gridElement2);
	void LoadBoundaryCondition(Parameters_structure::BoundaryCondition parameters, Matrix2D<double> gridElement1, Matrix2D<double> gridElement2);

	// Update the boundary conditions for current time step
	void Update(int iteration, Matrix2D<double> PSD_2D_Slice, double time=-1, double dt=-1);

	// Time-dependent boundary
	// A file with two columns is used: first column - is a time, second - filename of a file with an array of values for the boundary at a boundary
	// Time means we need to change the boundary value at that time
	vector <double> BC_change_time; ///< If we change boundary conditions with time, this is the time when we need to change it.
	vector <int> BC_change_iteration; ///< If we change boundary conditions with time, this is the iteration when we need to change it.
	vector <string> BC_filename; ///< If we change boundary conditions with time, this is the filename for the new boundary condition values.

};

#endif
