/*****************************************************************************/
/*                              cpplapack.h                                  */
/*****************************************************************************/

#ifndef CPPLAPACK_H
#define CPPLAPACK_H

//=============================================================================
#include <iostream>
#include <cstdlib>
#include <complex>
#include <cmath>
#include <vector>//only for ?geev, ?gegv, etc.
#include <fstream>
#include <string>

#ifdef  _MSC_VER
#include "prototype/VC++.h"
#endif//_MSC_VER

#include "prototype/dblas.h"
#include "prototype/zblas.h"
#include "prototype/dlapack.h"
#include "prototype/zlapack.h"

//=============================================================================
#ifndef CPPL_EPS
#define CPPL_EPS 1e-13
#endif//CPPL_EPS

#ifndef CPPL_SS_SECTOR
#define CPPL_SS_SECTOR 100
#endif//CPPL_SS_SECTOR

//=============================================================================
namespace CPPL{ //!< namespace for CPPLapack

//////////////// functions ////////////////
inline long min(long a, long b){ return ((a) <= (b) ? (a) : (b)); }
inline long max(long a, long b){ return ((a) >= (b) ? (a) : (b)); }

//////////////// class prototype ////////////////
class dgematrix; class _dgematrix;
class dgbmatrix; class _dgbmatrix;
class dsymatrix; class _dsymatrix;
class dssmatrix; class _dssmatrix;
class dcovector; class _dcovector;
class drovector; class _drovector;
//////////////////////////////////
class zgematrix; class _zgematrix;
class zgbmatrix; class _zgbmatrix;
class zhematrix; class _zhematrix;
class zssmatrix; class _zssmatrix;
class zcovector; class _zcovector;
class zrovector; class _zrovector;
//////////////////////////////////
class __zhecomplex;

//////////////// class definition ////////////////
/////////////////////////////////////
#include "dgematrix-/dgematrix.hpp"
#include "dgbmatrix-/dgbmatrix.hpp"
#include "dsymatrix-/dsymatrix.hpp"
#include "dssmatrix-/dssmatrix.hpp"
#include "dcovector-/dcovector.hpp"
#include "drovector-/drovector.hpp"
/////////////////////////////////////
#include "_dgematrix-/_dgematrix.hpp"
#include "_dgbmatrix-/_dgbmatrix.hpp"
#include "_dsymatrix-/_dsymatrix.hpp"
#include "_dssmatrix-/_dssmatrix.hpp"
#include "_dcovector-/_dcovector.hpp"
#include "_drovector-/_drovector.hpp"
/////////////////////////////////////
#include "zgematrix-/zgematrix.hpp"
#include "zgbmatrix-/zgbmatrix.hpp"
#include "zhematrix-/zhematrix.hpp"
#include "zssmatrix-/zssmatrix.hpp"
#include "zcovector-/zcovector.hpp"
#include "zrovector-/zrovector.hpp"
/////////////////////////////////////
#include "_zgematrix-/_zgematrix.hpp"
#include "_zgbmatrix-/_zgbmatrix.hpp"
#include "_zhematrix-/_zhematrix.hpp"
#include "_zssmatrix-/_zssmatrix.hpp"
#include "_zcovector-/_zcovector.hpp"
#include "_zrovector-/_zrovector.hpp"
/////////////////////////////////////
#include "__zhecomplex-/__zhecomplex.hpp"

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

//////////////////////// dgematrix ////////////////////////
#include "dgematrix-/dgematrix-constructor.hpp"
#include "dgematrix-/dgematrix-cast.hpp"
#include "dgematrix-/dgematrix-io.hpp"
#include "dgematrix-/dgematrix-misc.hpp"
#include "dgematrix-/dgematrix-calc.hpp"
#include "dgematrix-/dgematrix-lapack.hpp"
#include "dgematrix-/dgematrix-unary.hpp"
#include "dgematrix-/dgematrix-dgematrix.hpp"
#include "dgematrix-/dgematrix-_dgematrix.hpp"
#include "dgematrix-/dgematrix-dgbmatrix.hpp"
#include "dgematrix-/dgematrix-_dgbmatrix.hpp"
#include "dgematrix-/dgematrix-dsymatrix.hpp"
#include "dgematrix-/dgematrix-_dsymatrix.hpp"
#include "dgematrix-/dgematrix-dssmatrix.hpp"
#include "dgematrix-/dgematrix-_dssmatrix.hpp"
#include "dgematrix-/dgematrix-dcovector.hpp"
#include "dgematrix-/dgematrix-_dcovector.hpp"
#include "dgematrix-/dgematrix-drovector.hpp"
#include "dgematrix-/dgematrix-_drovector.hpp"
#include "dgematrix-/dgematrix-double.hpp"
//////////////////////// _dgematrix ////////////////////////
#include "_dgematrix-/_dgematrix-constructor.hpp"
#include "_dgematrix-/_dgematrix-cast.hpp"
#include "_dgematrix-/_dgematrix-io.hpp"
#include "_dgematrix-/_dgematrix-misc.hpp"
#include "_dgematrix-/_dgematrix-calc.hpp"
#include "_dgematrix-/_dgematrix-unary.hpp"
#include "_dgematrix-/_dgematrix-dgematrix.hpp"
#include "_dgematrix-/_dgematrix-_dgematrix.hpp"
#include "_dgematrix-/_dgematrix-dgbmatrix.hpp"
#include "_dgematrix-/_dgematrix-_dgbmatrix.hpp"
#include "_dgematrix-/_dgematrix-dsymatrix.hpp"
#include "_dgematrix-/_dgematrix-_dsymatrix.hpp"
#include "_dgematrix-/_dgematrix-dssmatrix.hpp"
#include "_dgematrix-/_dgematrix-_dssmatrix.hpp"
#include "_dgematrix-/_dgematrix-dcovector.hpp"
#include "_dgematrix-/_dgematrix-_dcovector.hpp"
#include "_dgematrix-/_dgematrix-drovector.hpp"
#include "_dgematrix-/_dgematrix-_drovector.hpp"
#include "_dgematrix-/_dgematrix-double.hpp"


//////////////////////// dgbmatrix ////////////////////////
#include "dgbmatrix-/dgbmatrix-constructor.hpp"
#include "dgbmatrix-/dgbmatrix-cast.hpp"
#include "dgbmatrix-/dgbmatrix-io.hpp"
#include "dgbmatrix-/dgbmatrix-misc.hpp"
#include "dgbmatrix-/dgbmatrix-calc.hpp"
#include "dgbmatrix-/dgbmatrix-lapack.hpp"
#include "dgbmatrix-/dgbmatrix-unary.hpp"
#include "dgbmatrix-/dgbmatrix-dgematrix.hpp"
#include "dgbmatrix-/dgbmatrix-_dgematrix.hpp"
#include "dgbmatrix-/dgbmatrix-dgbmatrix.hpp"
#include "dgbmatrix-/dgbmatrix-_dgbmatrix.hpp"
#include "dgbmatrix-/dgbmatrix-dsymatrix.hpp"
#include "dgbmatrix-/dgbmatrix-_dsymatrix.hpp"
#include "dgbmatrix-/dgbmatrix-dssmatrix.hpp"
#include "dgbmatrix-/dgbmatrix-_dssmatrix.hpp"
#include "dgbmatrix-/dgbmatrix-dcovector.hpp"
#include "dgbmatrix-/dgbmatrix-_dcovector.hpp"
#include "dgbmatrix-/dgbmatrix-drovector.hpp"
#include "dgbmatrix-/dgbmatrix-_drovector.hpp"
#include "dgbmatrix-/dgbmatrix-double.hpp"
//////////////////////// _dgbmatrix ////////////////////////
#include "_dgbmatrix-/_dgbmatrix-constructor.hpp"
#include "_dgbmatrix-/_dgbmatrix-cast.hpp"
#include "_dgbmatrix-/_dgbmatrix-io.hpp"
#include "_dgbmatrix-/_dgbmatrix-misc.hpp"
#include "_dgbmatrix-/_dgbmatrix-calc.hpp"
#include "_dgbmatrix-/_dgbmatrix-unary.hpp"
#include "_dgbmatrix-/_dgbmatrix-dgematrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-_dgematrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-dgbmatrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-_dgbmatrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-dsymatrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-_dsymatrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-dssmatrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-_dssmatrix.hpp"
#include "_dgbmatrix-/_dgbmatrix-dcovector.hpp"
#include "_dgbmatrix-/_dgbmatrix-_dcovector.hpp"
#include "_dgbmatrix-/_dgbmatrix-drovector.hpp"
#include "_dgbmatrix-/_dgbmatrix-_drovector.hpp"
#include "_dgbmatrix-/_dgbmatrix-double.hpp"


//////////////////////// dsymatrix ////////////////////////
#include "dsymatrix-/dsymatrix-constructor.hpp"
#include "dsymatrix-/dsymatrix-cast.hpp"
#include "dsymatrix-/dsymatrix-io.hpp"
#include "dsymatrix-/dsymatrix-misc.hpp"
#include "dsymatrix-/dsymatrix-calc.hpp"
#include "dsymatrix-/dsymatrix-lapack.hpp"
#include "dsymatrix-/dsymatrix-unary.hpp"
#include "dsymatrix-/dsymatrix-dgematrix.hpp"
#include "dsymatrix-/dsymatrix-_dgematrix.hpp"
#include "dsymatrix-/dsymatrix-dgbmatrix.hpp"
#include "dsymatrix-/dsymatrix-_dgbmatrix.hpp"
#include "dsymatrix-/dsymatrix-dsymatrix.hpp"
#include "dsymatrix-/dsymatrix-_dsymatrix.hpp"
#include "dsymatrix-/dsymatrix-dssmatrix.hpp"
#include "dsymatrix-/dsymatrix-_dssmatrix.hpp"
#include "dsymatrix-/dsymatrix-dcovector.hpp"
#include "dsymatrix-/dsymatrix-_dcovector.hpp"
#include "dsymatrix-/dsymatrix-drovector.hpp"
#include "dsymatrix-/dsymatrix-_drovector.hpp"
#include "dsymatrix-/dsymatrix-double.hpp"
//////////////////////// _dsymatrix ////////////////////////
#include "_dsymatrix-/_dsymatrix-constructor.hpp"
#include "_dsymatrix-/_dsymatrix-cast.hpp"
#include "_dsymatrix-/_dsymatrix-io.hpp"
#include "_dsymatrix-/_dsymatrix-misc.hpp"
#include "_dsymatrix-/_dsymatrix-calc.hpp"
#include "_dsymatrix-/_dsymatrix-unary.hpp"
#include "_dsymatrix-/_dsymatrix-dgematrix.hpp"
#include "_dsymatrix-/_dsymatrix-_dgematrix.hpp"
#include "_dsymatrix-/_dsymatrix-dgbmatrix.hpp"
#include "_dsymatrix-/_dsymatrix-_dgbmatrix.hpp"
#include "_dsymatrix-/_dsymatrix-dsymatrix.hpp"
#include "_dsymatrix-/_dsymatrix-_dsymatrix.hpp"
#include "_dsymatrix-/_dsymatrix-dssmatrix.hpp"
#include "_dsymatrix-/_dsymatrix-_dssmatrix.hpp"
#include "_dsymatrix-/_dsymatrix-dcovector.hpp"
#include "_dsymatrix-/_dsymatrix-_dcovector.hpp"
#include "_dsymatrix-/_dsymatrix-drovector.hpp"
#include "_dsymatrix-/_dsymatrix-_drovector.hpp"
#include "_dsymatrix-/_dsymatrix-double.hpp"

//////////////////////// dssmatrix ////////////////////////
#include "dssmatrix-/dssmatrix-constructor.hpp"
#include "dssmatrix-/dssmatrix-cast.hpp"
#include "dssmatrix-/dssmatrix-io.hpp"
#include "dssmatrix-/dssmatrix-misc.hpp"
#include "dssmatrix-/dssmatrix-calc.hpp"
#include "dssmatrix-/dssmatrix-unary.hpp"
#include "dssmatrix-/dssmatrix-dgematrix.hpp"
#include "dssmatrix-/dssmatrix-_dgematrix.hpp"
#include "dssmatrix-/dssmatrix-dgbmatrix.hpp"
#include "dssmatrix-/dssmatrix-_dgbmatrix.hpp"
#include "dssmatrix-/dssmatrix-dsymatrix.hpp"
#include "dssmatrix-/dssmatrix-_dsymatrix.hpp"
#include "dssmatrix-/dssmatrix-dssmatrix.hpp"
#include "dssmatrix-/dssmatrix-_dssmatrix.hpp"
#include "dssmatrix-/dssmatrix-dcovector.hpp"
#include "dssmatrix-/dssmatrix-_dcovector.hpp"
#include "dssmatrix-/dssmatrix-drovector.hpp"
#include "dssmatrix-/dssmatrix-_drovector.hpp"
#include "dssmatrix-/dssmatrix-double.hpp"
//////////////////////// _dssmatrix ////////////////////////
#include "_dssmatrix-/_dssmatrix-constructor.hpp"
#include "_dssmatrix-/_dssmatrix-cast.hpp"
#include "_dssmatrix-/_dssmatrix-io.hpp"
#include "_dssmatrix-/_dssmatrix-misc.hpp"
#include "_dssmatrix-/_dssmatrix-calc.hpp"
#include "_dssmatrix-/_dssmatrix-unary.hpp"
#include "_dssmatrix-/_dssmatrix-dgematrix.hpp"
#include "_dssmatrix-/_dssmatrix-_dgematrix.hpp"
#include "_dssmatrix-/_dssmatrix-dgbmatrix.hpp"
#include "_dssmatrix-/_dssmatrix-_dgbmatrix.hpp"
#include "_dssmatrix-/_dssmatrix-dsymatrix.hpp"
#include "_dssmatrix-/_dssmatrix-_dsymatrix.hpp"
#include "_dssmatrix-/_dssmatrix-dssmatrix.hpp"
#include "_dssmatrix-/_dssmatrix-_dssmatrix.hpp"
#include "_dssmatrix-/_dssmatrix-dcovector.hpp"
#include "_dssmatrix-/_dssmatrix-_dcovector.hpp"
#include "_dssmatrix-/_dssmatrix-drovector.hpp"
#include "_dssmatrix-/_dssmatrix-_drovector.hpp"
#include "_dssmatrix-/_dssmatrix-double.hpp"


//////////////////////// dcovector ////////////////////////
#include "dcovector-/dcovector-constructor.hpp"
#include "dcovector-/dcovector-cast.hpp"
#include "dcovector-/dcovector-io.hpp"
#include "dcovector-/dcovector-calc.hpp"
#include "dcovector-/dcovector-misc.hpp"
#include "dcovector-/dcovector-unary.hpp"
#include "dcovector-/dcovector-dgematrix.hpp"
#include "dcovector-/dcovector-_dgematrix.hpp"
#include "dcovector-/dcovector-dgbmatrix.hpp"
#include "dcovector-/dcovector-_dgbmatrix.hpp"
#include "dcovector-/dcovector-dsymatrix.hpp"
#include "dcovector-/dcovector-_dsymatrix.hpp"
#include "dcovector-/dcovector-dssmatrix.hpp"
#include "dcovector-/dcovector-_dssmatrix.hpp"
#include "dcovector-/dcovector-dcovector.hpp"
#include "dcovector-/dcovector-_dcovector.hpp"
#include "dcovector-/dcovector-drovector.hpp"
#include "dcovector-/dcovector-_drovector.hpp"
#include "dcovector-/dcovector-double.hpp"
//////////////////////// _dcovector ////////////////////////
#include "_dcovector-/_dcovector-constructor.hpp"
#include "_dcovector-/_dcovector-cast.hpp"
#include "_dcovector-/_dcovector-io.hpp"
#include "_dcovector-/_dcovector-calc.hpp"
#include "_dcovector-/_dcovector-misc.hpp"
#include "_dcovector-/_dcovector-unary.hpp"
#include "_dcovector-/_dcovector-dgematrix.hpp"
#include "_dcovector-/_dcovector-_dgematrix.hpp"
#include "_dcovector-/_dcovector-dgbmatrix.hpp"
#include "_dcovector-/_dcovector-_dgbmatrix.hpp"
#include "_dcovector-/_dcovector-dsymatrix.hpp"
#include "_dcovector-/_dcovector-_dsymatrix.hpp"
#include "_dcovector-/_dcovector-dssmatrix.hpp"
#include "_dcovector-/_dcovector-_dssmatrix.hpp"
#include "_dcovector-/_dcovector-dcovector.hpp"
#include "_dcovector-/_dcovector-_dcovector.hpp"
#include "_dcovector-/_dcovector-drovector.hpp"
#include "_dcovector-/_dcovector-_drovector.hpp"
#include "_dcovector-/_dcovector-double.hpp"

//////////////////////// drovector ////////////////////////
#include "drovector-/drovector-constructor.hpp"
#include "drovector-/drovector-cast.hpp"
#include "drovector-/drovector-io.hpp"
#include "drovector-/drovector-calc.hpp"
#include "drovector-/drovector-misc.hpp"
#include "drovector-/drovector-unary.hpp"
#include "drovector-/drovector-dgematrix.hpp"
#include "drovector-/drovector-_dgematrix.hpp"
#include "drovector-/drovector-dgbmatrix.hpp"
#include "drovector-/drovector-_dgbmatrix.hpp"
#include "drovector-/drovector-dsymatrix.hpp"
#include "drovector-/drovector-_dsymatrix.hpp"
#include "drovector-/drovector-dssmatrix.hpp"
#include "drovector-/drovector-_dssmatrix.hpp"
#include "drovector-/drovector-dcovector.hpp"
#include "drovector-/drovector-_dcovector.hpp"
#include "drovector-/drovector-drovector.hpp"
#include "drovector-/drovector-_drovector.hpp"
#include "drovector-/drovector-double.hpp"
//////////////////////// _drovector ////////////////////////
#include "_drovector-/_drovector-constructor.hpp"
#include "_drovector-/_drovector-cast.hpp"
#include "_drovector-/_drovector-io.hpp"
#include "_drovector-/_drovector-calc.hpp"
#include "_drovector-/_drovector-misc.hpp"
#include "_drovector-/_drovector-unary.hpp"
#include "_drovector-/_drovector-dgematrix.hpp"
#include "_drovector-/_drovector-_dgematrix.hpp"
#include "_drovector-/_drovector-dgbmatrix.hpp"
#include "_drovector-/_drovector-_dgbmatrix.hpp"
#include "_drovector-/_drovector-dsymatrix.hpp"
#include "_drovector-/_drovector-_dsymatrix.hpp"
#include "_drovector-/_drovector-dssmatrix.hpp"
#include "_drovector-/_drovector-_dssmatrix.hpp"
#include "_drovector-/_drovector-dcovector.hpp"
#include "_drovector-/_drovector-_dcovector.hpp"
#include "_drovector-/_drovector-drovector.hpp"
#include "_drovector-/_drovector-_drovector.hpp"
#include "_drovector-/_drovector-double.hpp"

//////////////////////// double ////////////////////////
#include "double-/double-dgematrix.hpp"
#include "double-/double-_dgematrix.hpp"
#include "double-/double-dgbmatrix.hpp"
#include "double-/double-_dgbmatrix.hpp"
#include "double-/double-dsymatrix.hpp"
#include "double-/double-_dsymatrix.hpp"
#include "double-/double-dssmatrix.hpp"
#include "double-/double-_dssmatrix.hpp"
#include "double-/double-dcovector.hpp"
#include "double-/double-_dcovector.hpp"
#include "double-/double-drovector.hpp"
#include "double-/double-_drovector.hpp"

///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////
///////////////////////////////////////////////////////////////////////////////

//////////////////////// zgematrix ////////////////////////
#include "zgematrix-/zgematrix-constructor.hpp"
#include "zgematrix-/zgematrix-io.hpp"
#include "zgematrix-/zgematrix-misc.hpp"
#include "zgematrix-/zgematrix-calc.hpp"
#include "zgematrix-/zgematrix-lapack.hpp"
#include "zgematrix-/zgematrix-unary.hpp"
#include "zgematrix-/zgematrix-zgematrix.hpp"
#include "zgematrix-/zgematrix-_zgematrix.hpp"
#include "zgematrix-/zgematrix-zgbmatrix.hpp"
#include "zgematrix-/zgematrix-_zgbmatrix.hpp"
#include "zgematrix-/zgematrix-zhematrix.hpp"
#include "zgematrix-/zgematrix-_zhematrix.hpp"
#include "zgematrix-/zgematrix-zssmatrix.hpp"
#include "zgematrix-/zgematrix-_zssmatrix.hpp"
#include "zgematrix-/zgematrix-zcovector.hpp"
#include "zgematrix-/zgematrix-_zcovector.hpp"
#include "zgematrix-/zgematrix-zrovector.hpp"
#include "zgematrix-/zgematrix-_zrovector.hpp"
#include "zgematrix-/zgematrix-double.hpp"
#include "zgematrix-/zgematrix-complex.hpp"
//////////////////////// _zgematrix ////////////////////////
#include "_zgematrix-/_zgematrix-constructor.hpp"
#include "_zgematrix-/_zgematrix-io.hpp"
#include "_zgematrix-/_zgematrix-misc.hpp"
#include "_zgematrix-/_zgematrix-calc.hpp"
#include "_zgematrix-/_zgematrix-unary.hpp"
#include "_zgematrix-/_zgematrix-zgematrix.hpp"
#include "_zgematrix-/_zgematrix-_zgematrix.hpp"
#include "_zgematrix-/_zgematrix-zgbmatrix.hpp"
#include "_zgematrix-/_zgematrix-_zgbmatrix.hpp"
#include "_zgematrix-/_zgematrix-zhematrix.hpp"
#include "_zgematrix-/_zgematrix-_zhematrix.hpp"
#include "_zgematrix-/_zgematrix-zssmatrix.hpp"
#include "_zgematrix-/_zgematrix-_zssmatrix.hpp"
#include "_zgematrix-/_zgematrix-zcovector.hpp"
#include "_zgematrix-/_zgematrix-_zcovector.hpp"
#include "_zgematrix-/_zgematrix-zrovector.hpp"
#include "_zgematrix-/_zgematrix-_zrovector.hpp"
#include "_zgematrix-/_zgematrix-double.hpp"
#include "_zgematrix-/_zgematrix-complex.hpp"


//////////////////////// zgbmatrix ////////////////////////
#include "zgbmatrix-/zgbmatrix-constructor.hpp"
#include "zgbmatrix-/zgbmatrix-io.hpp"
#include "zgbmatrix-/zgbmatrix-misc.hpp"
#include "zgbmatrix-/zgbmatrix-calc.hpp"
#include "zgbmatrix-/zgbmatrix-lapack.hpp"
#include "zgbmatrix-/zgbmatrix-unary.hpp"
#include "zgbmatrix-/zgbmatrix-zgematrix.hpp"
#include "zgbmatrix-/zgbmatrix-_zgematrix.hpp"
#include "zgbmatrix-/zgbmatrix-zgbmatrix.hpp"
#include "zgbmatrix-/zgbmatrix-_zgbmatrix.hpp"
#include "zgbmatrix-/zgbmatrix-zhematrix.hpp"
#include "zgbmatrix-/zgbmatrix-_zhematrix.hpp"
#include "zgbmatrix-/zgbmatrix-zssmatrix.hpp"
#include "zgbmatrix-/zgbmatrix-_zssmatrix.hpp"
#include "zgbmatrix-/zgbmatrix-zcovector.hpp"
#include "zgbmatrix-/zgbmatrix-_zcovector.hpp"
#include "zgbmatrix-/zgbmatrix-zrovector.hpp"
#include "zgbmatrix-/zgbmatrix-_zrovector.hpp"
#include "zgbmatrix-/zgbmatrix-double.hpp"
#include "zgbmatrix-/zgbmatrix-complex.hpp"
//////////////////////// _zgbmatrix ////////////////////////
#include "_zgbmatrix-/_zgbmatrix-constructor.hpp"
#include "_zgbmatrix-/_zgbmatrix-io.hpp"
#include "_zgbmatrix-/_zgbmatrix-misc.hpp"
#include "_zgbmatrix-/_zgbmatrix-calc.hpp"
#include "_zgbmatrix-/_zgbmatrix-unary.hpp"
#include "_zgbmatrix-/_zgbmatrix-zgematrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-_zgematrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-zgbmatrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-_zgbmatrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-zhematrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-_zhematrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-zssmatrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-_zssmatrix.hpp"
#include "_zgbmatrix-/_zgbmatrix-zcovector.hpp"
#include "_zgbmatrix-/_zgbmatrix-_zcovector.hpp"
#include "_zgbmatrix-/_zgbmatrix-zrovector.hpp"
#include "_zgbmatrix-/_zgbmatrix-_zrovector.hpp"
#include "_zgbmatrix-/_zgbmatrix-double.hpp"
#include "_zgbmatrix-/_zgbmatrix-complex.hpp"


//////////////////////// zhematrix ////////////////////////
#include "zhematrix-/zhematrix-constructor.hpp"
#include "zhematrix-/zhematrix-io.hpp"
#include "zhematrix-/zhematrix-misc.hpp"
#include "zhematrix-/zhematrix-calc.hpp"
#include "zhematrix-/zhematrix-lapack.hpp"
#include "zhematrix-/zhematrix-unary.hpp"
#include "zhematrix-/zhematrix-zgematrix.hpp"
#include "zhematrix-/zhematrix-_zgematrix.hpp"
#include "zhematrix-/zhematrix-zgbmatrix.hpp"
#include "zhematrix-/zhematrix-_zgbmatrix.hpp"
#include "zhematrix-/zhematrix-zhematrix.hpp"
#include "zhematrix-/zhematrix-_zhematrix.hpp"
#include "zhematrix-/zhematrix-zssmatrix.hpp"
#include "zhematrix-/zhematrix-_zssmatrix.hpp"
#include "zhematrix-/zhematrix-zcovector.hpp"
#include "zhematrix-/zhematrix-_zcovector.hpp"
#include "zhematrix-/zhematrix-zrovector.hpp"
#include "zhematrix-/zhematrix-_zrovector.hpp"
#include "zhematrix-/zhematrix-double.hpp"
#include "zhematrix-/zhematrix-complex.hpp"
//////////////////////// _zhematrix ////////////////////////
#include "_zhematrix-/_zhematrix-constructor.hpp"
#include "_zhematrix-/_zhematrix-io.hpp"
#include "_zhematrix-/_zhematrix-misc.hpp"
#include "_zhematrix-/_zhematrix-calc.hpp"
#include "_zhematrix-/_zhematrix-unary.hpp"
#include "_zhematrix-/_zhematrix-zgematrix.hpp"
#include "_zhematrix-/_zhematrix-_zgematrix.hpp"
#include "_zhematrix-/_zhematrix-zgbmatrix.hpp"
#include "_zhematrix-/_zhematrix-_zgbmatrix.hpp"
#include "_zhematrix-/_zhematrix-zhematrix.hpp"
#include "_zhematrix-/_zhematrix-_zhematrix.hpp"
#include "_zhematrix-/_zhematrix-zssmatrix.hpp"
#include "_zhematrix-/_zhematrix-_zssmatrix.hpp"
#include "_zhematrix-/_zhematrix-zcovector.hpp"
#include "_zhematrix-/_zhematrix-_zcovector.hpp"
#include "_zhematrix-/_zhematrix-zrovector.hpp"
#include "_zhematrix-/_zhematrix-_zrovector.hpp"
#include "_zhematrix-/_zhematrix-double.hpp"
#include "_zhematrix-/_zhematrix-complex.hpp"


//////////////////////// zssmatrix ////////////////////////
#include "zssmatrix-/zssmatrix-constructor.hpp"
#include "zssmatrix-/zssmatrix-io.hpp"
#include "zssmatrix-/zssmatrix-misc.hpp"
#include "zssmatrix-/zssmatrix-calc.hpp"
#include "zssmatrix-/zssmatrix-unary.hpp"
#include "zssmatrix-/zssmatrix-zgematrix.hpp"
#include "zssmatrix-/zssmatrix-_zgematrix.hpp"
#include "zssmatrix-/zssmatrix-zgbmatrix.hpp"
#include "zssmatrix-/zssmatrix-_zgbmatrix.hpp"
#include "zssmatrix-/zssmatrix-zhematrix.hpp"
#include "zssmatrix-/zssmatrix-_zhematrix.hpp"
#include "zssmatrix-/zssmatrix-zssmatrix.hpp"
#include "zssmatrix-/zssmatrix-_zssmatrix.hpp"
#include "zssmatrix-/zssmatrix-zcovector.hpp"
#include "zssmatrix-/zssmatrix-_zcovector.hpp"
#include "zssmatrix-/zssmatrix-zrovector.hpp"
#include "zssmatrix-/zssmatrix-_zrovector.hpp"
#include "zssmatrix-/zssmatrix-double.hpp"
#include "zssmatrix-/zssmatrix-complex.hpp"
//////////////////////// _zssmatrix ////////////////////////
#include "_zssmatrix-/_zssmatrix-constructor.hpp"
#include "_zssmatrix-/_zssmatrix-io.hpp"
#include "_zssmatrix-/_zssmatrix-misc.hpp"
#include "_zssmatrix-/_zssmatrix-calc.hpp"
#include "_zssmatrix-/_zssmatrix-unary.hpp"
#include "_zssmatrix-/_zssmatrix-zgematrix.hpp"
#include "_zssmatrix-/_zssmatrix-_zgematrix.hpp"
#include "_zssmatrix-/_zssmatrix-zgbmatrix.hpp"
#include "_zssmatrix-/_zssmatrix-_zgbmatrix.hpp"
#include "_zssmatrix-/_zssmatrix-zhematrix.hpp"
#include "_zssmatrix-/_zssmatrix-_zhematrix.hpp"
#include "_zssmatrix-/_zssmatrix-zssmatrix.hpp"
#include "_zssmatrix-/_zssmatrix-_zssmatrix.hpp"
#include "_zssmatrix-/_zssmatrix-zcovector.hpp"
#include "_zssmatrix-/_zssmatrix-_zcovector.hpp"
#include "_zssmatrix-/_zssmatrix-zrovector.hpp"
#include "_zssmatrix-/_zssmatrix-_zrovector.hpp"
#include "_zssmatrix-/_zssmatrix-double.hpp"
#include "_zssmatrix-/_zssmatrix-complex.hpp"


//////////////////////// zcovector ////////////////////////
#include "zcovector-/zcovector-constructor.hpp"
#include "zcovector-/zcovector-io.hpp"
#include "zcovector-/zcovector-calc.hpp"
#include "zcovector-/zcovector-misc.hpp"
#include "zcovector-/zcovector-unary.hpp"
#include "zcovector-/zcovector-zgematrix.hpp"
#include "zcovector-/zcovector-_zgematrix.hpp"
#include "zcovector-/zcovector-zgbmatrix.hpp"
#include "zcovector-/zcovector-_zgbmatrix.hpp"
#include "zcovector-/zcovector-zhematrix.hpp"
#include "zcovector-/zcovector-_zhematrix.hpp"
#include "zcovector-/zcovector-zssmatrix.hpp"
#include "zcovector-/zcovector-_zssmatrix.hpp"
#include "zcovector-/zcovector-zcovector.hpp"
#include "zcovector-/zcovector-_zcovector.hpp"
#include "zcovector-/zcovector-zrovector.hpp"
#include "zcovector-/zcovector-_zrovector.hpp"
#include "zcovector-/zcovector-double.hpp"
#include "zcovector-/zcovector-complex.hpp"
//////////////////////// _zcovector ////////////////////////
#include "_zcovector-/_zcovector-constructor.hpp"
#include "_zcovector-/_zcovector-io.hpp"
#include "_zcovector-/_zcovector-calc.hpp"
#include "_zcovector-/_zcovector-misc.hpp"
#include "_zcovector-/_zcovector-unary.hpp"
#include "_zcovector-/_zcovector-zgematrix.hpp"
#include "_zcovector-/_zcovector-_zgematrix.hpp"
#include "_zcovector-/_zcovector-zgbmatrix.hpp"
#include "_zcovector-/_zcovector-_zgbmatrix.hpp"
#include "_zcovector-/_zcovector-zhematrix.hpp"
#include "_zcovector-/_zcovector-_zhematrix.hpp"
#include "_zcovector-/_zcovector-zssmatrix.hpp"
#include "_zcovector-/_zcovector-_zssmatrix.hpp"
#include "_zcovector-/_zcovector-zcovector.hpp"
#include "_zcovector-/_zcovector-_zcovector.hpp"
#include "_zcovector-/_zcovector-zrovector.hpp"
#include "_zcovector-/_zcovector-_zrovector.hpp"
#include "_zcovector-/_zcovector-double.hpp"
#include "_zcovector-/_zcovector-complex.hpp"

//////////////////////// zrovector ////////////////////////
#include "zrovector-/zrovector-constructor.hpp"
#include "zrovector-/zrovector-io.hpp"
#include "zrovector-/zrovector-calc.hpp"
#include "zrovector-/zrovector-misc.hpp"
#include "zrovector-/zrovector-unary.hpp"
#include "zrovector-/zrovector-zgematrix.hpp"
#include "zrovector-/zrovector-_zgematrix.hpp"
#include "zrovector-/zrovector-zgbmatrix.hpp"
#include "zrovector-/zrovector-_zgbmatrix.hpp"
#include "zrovector-/zrovector-zhematrix.hpp"
#include "zrovector-/zrovector-_zhematrix.hpp"
#include "zrovector-/zrovector-zssmatrix.hpp"
#include "zrovector-/zrovector-_zssmatrix.hpp"
#include "zrovector-/zrovector-zcovector.hpp"
#include "zrovector-/zrovector-_zcovector.hpp"
#include "zrovector-/zrovector-zrovector.hpp"
#include "zrovector-/zrovector-_zrovector.hpp"
#include "zrovector-/zrovector-double.hpp"
#include "zrovector-/zrovector-complex.hpp"
//////////////////////// _zrovector ////////////////////////
#include "_zrovector-/_zrovector-constructor.hpp"
#include "_zrovector-/_zrovector-io.hpp"
#include "_zrovector-/_zrovector-calc.hpp"
#include "_zrovector-/_zrovector-misc.hpp"
#include "_zrovector-/_zrovector-unary.hpp"
#include "_zrovector-/_zrovector-zgematrix.hpp"
#include "_zrovector-/_zrovector-_zgematrix.hpp"
#include "_zrovector-/_zrovector-zgbmatrix.hpp"
#include "_zrovector-/_zrovector-_zgbmatrix.hpp"
#include "_zrovector-/_zrovector-zhematrix.hpp"
#include "_zrovector-/_zrovector-_zhematrix.hpp"
#include "_zrovector-/_zrovector-zssmatrix.hpp"
#include "_zrovector-/_zrovector-_zssmatrix.hpp"
#include "_zrovector-/_zrovector-zcovector.hpp"
#include "_zrovector-/_zrovector-_zcovector.hpp"
#include "_zrovector-/_zrovector-zrovector.hpp"
#include "_zrovector-/_zrovector-_zrovector.hpp"
#include "_zrovector-/_zrovector-double.hpp"
#include "_zrovector-/_zrovector-complex.hpp"

//////////////////////// double ////////////////////////
#include "double-/double-zgematrix.hpp"
#include "double-/double-_zgematrix.hpp"
#include "double-/double-zgbmatrix.hpp"
#include "double-/double-_zgbmatrix.hpp"
#include "double-/double-zhematrix.hpp"
#include "double-/double-_zhematrix.hpp"
#include "double-/double-zssmatrix.hpp"
#include "double-/double-_zssmatrix.hpp"
#include "double-/double-zcovector.hpp"
#include "double-/double-_zcovector.hpp"
#include "double-/double-zrovector.hpp"
#include "double-/double-_zrovector.hpp"

//////////////////////// complex ////////////////////////
#include "complex-/complex-zgematrix.hpp"
#include "complex-/complex-_zgematrix.hpp"
#include "complex-/complex-zgbmatrix.hpp"
#include "complex-/complex-_zgbmatrix.hpp"
#include "complex-/complex-zhematrix.hpp"
#include "complex-/complex-_zhematrix.hpp"
#include "complex-/complex-zssmatrix.hpp"
#include "complex-/complex-_zssmatrix.hpp"
#include "complex-/complex-zcovector.hpp"
#include "complex-/complex-_zcovector.hpp"
#include "complex-/complex-zrovector.hpp"
#include "complex-/complex-_zrovector.hpp"


/*****************************************************************************/
}//namespace CPPL
#endif//CPPLAPACK_H
