/**
 * \file Output.h
 * \brief Logging and screen output. see namespace Output for details
 *
 * The file provide output functions. Output usually goes to a screen and a file
 * simultaniously.
 *
 * \todo 
 * - Find out what is the maximum level of the message
 *
 * \author Developed under supervision of the PI Yuri Shprits
 */

#ifndef OUTPUT_H

/// \def OUTPUT_H
/// \brief Check if OUTPUT_H is defined, to not include the class more then one time.
#define OUTPUT_H

#include <string.h>
#include <stdarg.h>
#include <sstream>
#include <fstream>
#include <iostream>

// deprecated ??? (it was defined in main.cpp)
#include <ctime>
#include <time.h>

// include functions, provided exceptions handling
#include "../Exceptions/error.h"

/**
* \namespace Output
* \brief functions for write log and support files. Functions are defined in Output.h and descripted in Output.cpp
*/
namespace Output {

	
	/** \brief Open file stream for log file
	*	\param filename - name of log file	*/
	void open_log_file(string filename);

	/** \brief Close file stream for log file */
	void close_log_file();
	
	/** \brief flush file stream for log file */
	void flush_log_file();
	
	/** \brief Basic function! Write the message to the file
	*	\param msglvl - verbose level for the message from 0 to 3 
	*	\param format - sprintf format for the output string	
	*	\param ... - variables used for format string	*/
	void echo(int msglvl, const char * format, ... );
	
	/** \brief Set the verbose level until redefine
	*	\param new_outputLvl - verbose level */
	void set_output_lvl(int new_outputLvl);

}

#endif
