/*
 * Akima.cpp
 *
 *  Created on: Sep 3, 2010
 *      Author: subbotin
 */

#include "akima.h"
#include <iostream>
#include <stdlib.h>

using namespace std;

#define MERR 1e-19


bool akima(double *x0, double *y0, int nx, double *xi,double *yi, int nxi, int extrapolation_type, double ub, double lb, double ub_x, double lb_x){

	double max_f,x_lower,y_lower,x_upper,y_upper;
	int exp_lower,exp_upper;

	double* x=new double[nx+2];
	double* y=new double[nx+2];
	double* dx=new double[nx+2];
	double* dy=new double[nx+2];
	double* diff=new double[nx+6];
	double* ds=new double[nx+5];
	double* f=new double[nx+2];
	double* f1=new double[nx+2];
	double* f2=new double[nx+2];
	double* b=new double[nx+2];
	double* c=new double[nx+2];
	double* d=new double[nx+2];
	double* wj=new double[nxi];
	int* max_id=new int[nx+2];
	int* index=new int[nxi];
	int* bb=new int[nxi];

	/////////////////////////////////////////////////
	//	cout<<input x and y arrays must be of same length<<endl;
	/////////////////////////////////////////////////

	if (extrapolation_type == 0) {
		////////// linear extrapolation //////////////
		if(x0[0]>xi[0]) {
			y_lower=y0[0]-(y0[1]-y0[0])/(x0[1]-x0[0])*(x0[0]-xi[0]);
			x_lower=x0[0]-(x0[0]-xi[0]);
			exp_lower=1;
		} else {
			exp_lower=0;
		}
		if(x0[nx-1]<xi[nxi-1]) {
			y_upper=y0[nx-1]+(y0[nx-1]-y0[nx-2])/(x0[nx-1]-x0[nx-2])*(xi[nxi-1]-x0[nx-1]);
			x_upper=x0[nx-1]+(xi[nxi-1]-x0[nx-1]);
			exp_upper=1;
		} else {
			exp_upper=0;
		}
	} else if (extrapolation_type == 1) {
		////////// fixed boundaries //////////////
		if(x0[0]>xi[0]) {
			y_lower=lb;
			x_lower=x0[0]-(x0[0]-xi[0]);
			exp_lower=1;
		} else {
			exp_lower=0;
		}
		if(x0[nx-1]<xi[nxi-1]) {
			y_upper=ub;
			x_upper=x0[nx-1]+(xi[nxi-1]-x0[nx-1]);
			exp_upper=1;
		} else {
			exp_upper=0;
		}
	} else if (extrapolation_type == 2) {
		////////// fixed boundaries at fixed x //////////////
		y_lower=lb;
		x_lower=lb_x;
		exp_lower=1;
		y_upper=ub;
		x_upper=ub_x;
	} else {
		////////// no extrapolation  //////////////
		if(x0[0]>xi[0]) {
			cout << "extrapolation" << endl;
			exit(0);
		} else {
			exp_lower=0;
		}
		if(x0[nx-1]<xi[nxi-1]) {
			cout << "extrapolation" << endl;
			exit(0);
		} else {
			exp_upper=0;
		}
	}

	///// expand an input array x0 by adding two extrapolated boundary conditions /////////
	if(exp_lower==1 && exp_upper==1) {
		x[0]=x_lower;
		y[0]=y_lower;
		for(int i=0;i<nx;i++) {
			x[i+1]=x0[i];
			y[i+1]=y0[i];
		}
		x[nx+1]=x_upper;
		y[nx+1]=y_upper;
		nx=nx+2;
	} else if(exp_lower==1 && exp_upper==0) {
			x[0]=x_lower;
			y[0]=y_lower;
			for(int i=0;i<nx;i++) {
				x[i+1]=x0[i];
				y[i+1]=y0[i];
			}
			nx=nx+1;
	} else if(exp_lower==0 && exp_upper==1){
			for(int i=0;i<nx;i++) {
				x[i]=x0[i];
				y[i]=y0[i];
			}
			x[nx]=x_upper;
			y[nx]=y_upper;
			nx=nx+1;
	} else {
		for(int i=0;i<nx;i++) {
			x[i]=x0[i];
			y[i]=y0[i];
		}
		nx=nx;
	}
	///////////////////////////////////////

	///////////////// slopes //////////////////////
	for (int i=0;i<nx-1;i++) {  // 0 to nx-2
		dx[i]=x[i+1]-x[i];
		dy[i]=y[i+1]-y[i];
		diff[i+2]=dy[i]/dx[i];
		if(dx[i]<0.0){
			cout<<"input x-array must be in strictly ascending order"<<endl;
			//return false;
			exit(0);

		}
	}

	//diff[i+2]=(y[i+1]-y[i])/(x[i+1]-x[i]);


	diff[1]=2.0*diff[2]-diff[3];
	diff[0]=2.0*diff[1]-diff[2];
	diff[nx+1]=2.0*diff[nx]-diff[nx-1];
	diff[nx+2]=2.0*diff[nx+1]-diff[nx];
	/////////////////////////////////////////////


	for (int i=0;i<nx+2;i++) {
		ds[i]=fabs(diff[i+1]-diff[i]);
	}

	for (int i=0;i<nx;i++) {
		f1[i]=ds[i+2];
		f2[i]=ds[i];
		f[i]=f1[i]+f2[i];
	///////////////////////////////////
		b[i]=diff[i+1];
	///////////////////////////////////
	}

	// find the maximum value of f
	max_f=f[0];
	for (int i=0;i<nx;i++) {
		if(f[i]>max_f){
			max_f=f[i];
		}
	}


	int j=0;
	int max_count=0;
	for (int i=0;i<nx;i++) {
		if(f[i]>max_f*1e-8){
			max_id[j]=i;
			j=j+1;
			max_count=j;
		}
	}

	/*
	for (int i=0;i<nx;i++) {
		b[i]=diff[i+1];
	}
	*/

	if(max_count!=0){
		for (int i=0;i<max_count;i++) {
			b[max_id[i]]=( f1[max_id[i]]*diff[max_id[i]+1]+f2[max_id[i]]*diff[max_id[i]+2] ) / f[max_id[i]];
		}
	}


	for (int i=0;i<nx-1;i++) {
		c[i]=(3.0*diff[i+2]-2.0*b[i]-b[i+1])/dx[i];
		d[i]=(b[i]+b[i+1]-2.0*diff[i+2])/pow(dx[i],2);
	}

	///////////////// data binning ////////////////////
	int i=0;
	for(int j=0;j<nxi;j++){
		if(xi[j]>=x[i] && xi[j]<x[i+1]){
			index[j]=i;
			//cout<<i<<" "<<x[i]<<"  "<<x[i+1]<<" "<<xi[j]<<" "<<endl;
		} else {
			if(i==nx-1) {
				for(int k=j-1;k<nxi;k++) {
					index[k]=i;
				//	cout<<k<<" "<<xi[k]<<" "<<index[k]<<endl;
				}
				break;
			} else {
				i++;
				j--;
			}
		}
	}


	/*
	for (int i=0;i<nxi;i++) {
		if(index[i]>nx-2) {
			index[i]=nx-2;
		}
		//cout<<nx-1<<" index "<<index[i]<<endl;
	}
	*/

	///////////////////////////


	for (int i=0;i<nxi;i++) {
		//
		if(index[i]>nx-2) {
			index[i]=nx-2;
		}
		//
		bb[i]=index[i];
		wj[i]=xi[i]-x[bb[i]];
		yi[i]=( (wj[i]*d[bb[i]]+c[bb[i]])*wj[i]+b[bb[i]] )
			    *wj[i]+y[bb[i]];
	}

	delete []x;
	delete []y;
	delete []dx;
	delete []dy;
	delete []diff;
	delete []ds;
	delete []f;
	delete []f1;
	delete []f2;
	delete []b;
	delete []c;
	delete []d;
	delete []index;
	delete []bb;
	delete []wj;

	return true;
}







/**
 *  Akima
 *  /param s - function
 *  /param x - grid
 *  /param k - number of points
 */

double akima2(double *x, double *y, int nx, double *xx, double *yy, int nnx, double ub, double lb) {
	int i, ii;

	// derivatives
	double* d = new double[nx];
	double* w = new double[nx];
	double* dx = new double[nx];
	double* dy = new double[nx];

	double* p0 = new double[nx];
	double* p1 = new double[nx];
	double* p2 = new double[nx];
	double* p3 = new double[nx];

	double* yder = new double[nx];

	double dm1, dm2, dnx, wm1, wnx;

	for (i = 0; i <= nx-2; i++) {
		dx[i] = x[i+1] - x[i];
		dy[i] = y[i+1] - y[i];
	}

	for (i = 0; i <= nx-2; i++) {
		d[i] = dy[i]/dx[i]; // d defined for i = 0 .. k-2
	}

	dm1 = 2 * d[0] - d[1];
	dm2 = 2 * dm1 - d[0];
	d[nx-1] = 2 * d[nx-2] - d[nx-3];
	dnx = 2 * d[nx-1] - d[nx-2];

	for (i = 1; i <= nx-2; i++) {
		w[i] = fabs(d[i] - d[i-1]); // w defined for i = 1 .. k-2 !!!
	}
	w[0] = fabs(d[0] - dm1);
	wm1 = fabs(dm1 - dm2);
	w[nx-1] = fabs(d[nx-1] - d[nx-2]);
	wnx =fabs(dnx - d[nx-1]);

	//t = (|m4 - m3|*m2 + |m2 - m1|*m3)/(|m4 - m3| + |m2 - m1|)
	//t = (|d[4] - d[3]|*d[2] + |d[2] - d[1]|*d[3])/(|d[4] - d[3]| + |d[2] - d[1]|)
	//t3 = (|d[i+1] - d[i]|*d[i-1] + |d[i-1] - d[i-2]|*d[i])/(|d[i+1] - d[i]| + |d[i-1] - d[i-2]|)
	//t3 = (|w[i+1]|*d[i-1] + |w[i-1]|*d[i])/(|w[i+1]| + |w[i-1]|)
	//t3 = (w[i+1]*d[i-1] + w[i-1]*d[i])/(w[i+1] + w[i-1]);

	for (i = 2; i <= nx-3; i++) {
		if (fabs(d[i-2] - d[i-1]) < MERR && fabs(d[i] - d[i+1]) < MERR && fabs(d[i-1] - d[i]) > MERR) {
			yder[i] = 0.5 * (d[i-1] + d[i]);
		} else if (fabs(d[i-2] - d[i-1]) < MERR) {
			yder[i] = d[i-1];
		} else if (fabs(d[i+1] - d[i]) < MERR) {
			yder[i] = d[i];
		} else {
			yder[i] = (w[i+1]*d[i-1] + w[i-1]*d[i])/(w[i+1] + w[i-1]);
		}
	}

	if (fabs(dm2 - dm1) < MERR && fabs(d[0] - d[1]) < MERR && fabs(dm1 - d[0]) > MERR) {
		yder[0] = 0.5 * (dm1 + d[0]);
	} else if (fabs(dm2 - dm1) < MERR) {
		yder[0] = dm1;
	} else if (fabs(d[1] - d[0]) < MERR) {
		yder[0] = d[0];
	} else {
		yder[0] = (w[1]*dm1 + wm1*d[0])/(w[1] + wm1);
	}

	if (fabs(dm1 - d[0]) < MERR && fabs(d[1] - d[2]) < MERR && fabs(d[0] - d[1]) > MERR) {
		yder[1] = 0.5 * (d[0] + d[1]);
	} else if (fabs(dm1 - d[0]) < MERR) {
		yder[1] = d[0];
	} else if (fabs(d[2] - d[1]) < MERR) {
		yder[1] = d[1];
	} else {
		yder[1] = (w[2]*d[0] + w[0]*d[1])/(w[2] + w[0]);
	}

	if (fabs(d[nx-3] - d[nx-2]) < MERR && fabs(d[nx-1] - dnx) < MERR && fabs(d[nx-2] - d[nx-1]) > MERR) {
		yder[nx-1] = 0.5 * (d[nx-2] + d[nx-1]);
	} else if (fabs(d[nx-3] - d[nx-2]) < MERR) {
		yder[nx-1] = d[nx-2];
	} else if (fabs(dnx - d[nx-1]) < MERR) {
		yder[nx-1] = d[nx-1];
	} else {
		yder[nx-1] = (wnx*d[nx-2] + w[nx-2]*d[nx-1])/(wnx + w[nx-2]);
	}

	if (fabs(d[nx-4] - d[nx-3]) < MERR && fabs(d[nx-2] - d[nx-1]) < MERR && fabs(d[nx-3] - d[nx-2]) > MERR) {
		yder[nx-2] = 0.5 * (d[nx-3] + d[nx-2]);
	} else if (fabs(d[nx-4] - d[nx-3]) < MERR) {
		yder[nx-2] = d[nx-3];
	} else if (fabs(d[nx-1] - d[nx-2]) < MERR) {
		yder[nx-2] = d[nx-2];
	} else {
		yder[nx-2] = (w[nx-1]*d[nx-3] + w[nx-3]*d[nx-2])/(w[nx-1] + w[nx-3]);
	}

	//for (i = 2; i <= nx-3; i++) {
	for (i = 0; i <= nx-2; i++) {
		p0[i] = y[i];
		p1[i] = yder[i];
		p2[i] = (3*d[i] - 2*yder[i] - yder[i+1]) / dx[i];
		p3[i] = (yder[i] + yder[i+1] - 2*d[i]) / dx[i] / dx[i];
		//p2[i] = (3*d[i] - 2*w[i] - w[i+1]) / dx[i];
		//p3[i] = (w[i] + w[i+1] - 2*d[i]) / dx[i] / dx[i];
	}

	i = 0;
	// assume grids values are 'sorted'
	for (ii = 0; ii <= nnx-1; ii++) {
		while (i <= nx-2 && x[i+1] <= xx[ii]) i++;
		if (fabs(x[i] - xx[ii]) < MERR) {
			yy[ii] = y[i];
		} else 	if (fabs(x[i+1] - xx[ii]) < MERR) {
			yy[ii] = y[i+1];
		} else if (xx[ii] > x[i] && xx[ii] < x[i+1]) {
			yy[ii] = p0[i] + p1[i]*(xx[ii] - x[i]) + p2[i]*pow((xx[ii] - x[i]), 2) + p3[i]*pow((xx[ii] - x[i]), 3);
		} else {
			// something wrong
			exit(0);
		}
	}


	return -1;
}


