/**
 * Linear interpolation
 * \file linear.h
 */

#ifndef MATHS_INTERPOLATION_LINEAR_H
#define MATHS_INTERPOLATION_LINEAR_H

namespace Maths
{
	namespace Interpolation
	{
		/** 
         * \brief %Linear interpolation of two arrays.
		 */
		class Linear {
		public:
            /// size of matrix
			int size;
            
            /// Linear interpolation constructor
            /// \param n - length of arrays
            /// \param *x - pointer to start of array x
            /// \param *y - pointer to start of array y
			Linear(int n, double *x, double *y) {
				size = n;
				m_x = new double[n];
				m_y = new double[n];

				for (int i = 0; i < n; ++i) {
					m_x[i] = x[i];
					m_y[i] = y[i];
				}

			}

            /// Linear interpolation destructor
            /// free memory of m_x and m_y
			~Linear() {

				delete [] m_x;
				delete [] m_y;

			}
            /// Obtains the value from interpolating the two arrays nearest to x
            /// \param x - value for which to interpolate at
			double getValue(double x) {
				int i = 0;
				while (x > m_x[++i] && i < size-1);

				double a = (x - m_x[i - 1]) / (m_x[i] - m_x[i - 1]);
				return m_y[i - 1] + a * (m_y[i] - m_y[i - 1]);
			}

		private:
            ///private copies of arrays to interpolate
			double *m_x, *m_y;
		};
	};
};

#endif
