/**
 * Polilinear interpolation sources.
 * \file polilinear.cpp
 * \author Y.Shprits
 */

#include "../../Exceptions/error.h"

/** Do polinomial interpolation between 3 points for all data exepts the boundaries.
 * Linear interpolation on the boundaries.
 * Boundary values for extrapolation.
 */
double polilinear( double *xa, double *ya, int n, double x, double lb, double ub)
{
	int i = 0;

	while (i <= n-1 && x > xa[i]) {
		i = i + 1;
	}

//	if ((x <= xa[0])) {
	if (i <= 0) {
		return lb;
	} else if (i <= 1) {
		// linear
		double a = (x - xa[i-1])/(xa[i] - xa[i-1]);
		return ya[i-1] + a*(ya[i] - ya[i-1]);
	} else if (i >= n-1) {
		return ub;
//		return -21;
	} else if (i >= n-2) {
		// linear
		double a = (x - xa[i-1])/(xa[i] - xa[i-1]);
		return ya[i-1] + a*(ya[i] - ya[i-1]);
//		double a = (x - xa[i])/(xa[i-1] - xa[i]);
//		return ya[i] + a*(ya[i-1] - ya[i]);
/*	} else if (i == n-1) {
		//polinom
		double A = (x - xa[i-1])*(x - xa[i]) / (xa[i-2] - xa[i-1]) / (xa[i-2] - xa[i]);
		double B = (xa[i-2] - x)*(x - xa[i]) / (xa[i-2] - xa[i-1]) / (xa[i-1] - xa[i]);
		double C = (xa[i-2] - x)*(xa[i-1] - x) / (xa[i-1] - xa[i]) / (xa[i-2] - xa[i]);
		return A*ya[i-2] + B*ya[i-1] + C*ya[i];*/
	} else if (i >= 1 && i <= n-2) {
		//polinom
		double A1 = (x - xa[i])*(x - xa[i+1]) / (xa[i-1] - xa[i]) / (xa[i-1] - xa[i+1]);
		double B1 = (xa[i-1] - x)*(x - xa[i+1]) / (xa[i-1] - xa[i]) / (xa[i] - xa[i+1]);
		double C1 = (xa[i-1] - x)*(xa[i] - x) / (xa[i] - xa[i+1]) / (xa[i-1] - xa[i+1]);
		double res = A1*ya[i-1] + B1*ya[i] + C1*ya[i+1];
//		if (res > 99 || (!(res >= 0) && !(res <= 0))) {
//			cout << "111";
//		}
		return res;
	} else {
		//cout << "interpolation error";
		//exit(0);
		throw error_msg("POLINOM_ERROR", "Polinomial interpolation error");
	}
}
