/**
* \file Matrix.cpp
*
* \author Developed under supervision of the PI Yuri Shprits
*/
#ifndef matrix_array_MATRIX_CPP
#define matrix_array_MATRIX_CPP

#include "Matrix.h"

using namespace std;

#if defined(_WIN32) || defined(_WIN64)
#define strncasecmp _strnicmp
#define strcasecmp _stricmp
#endif

// Memory related functions

/// Allocating memory for 1D matrix
template<class T>inline T* matrix(long Rows)
{
	T *m=new T[Rows];
	//	assert(m!=NULL);
	if (m == NULL) {
		throw error_msg("MEMORY_ERROR", "Memory can't be initialized: %s size", Rows*sizeof(T));
	}
	return m;
}

/// Initilizing memory for 2D matrix
template<class T>inline T** matrix(long Rows, long Columns)
{
	// allocating memory for array of pinters
	T **m=new T*[Rows];
	//	assert(m!=NULL);
	if (m == NULL) {
		throw error_msg("MEMORY_ERROR", "Memory can't be initialized: %s size", Rows * sizeof(T));
	}
	// allocating memory for data array
	m[0] = new T[Rows * Columns];
	//	assert(m[0]!=NULL);
	if (m[0] == NULL) {
		throw error_msg("MEMORY_ERROR", "Memory can't be initialized: %s size", Rows * Columns * sizeof(T));
	}
	// assign pointers to data ranges
	for(long i=1; i<Rows; i++) m[i] = m[i-1] + Columns;
	return m;
}

/// Initializing memory for 3D matrix
template<class T>inline T*** matrix(int size_x, int size_y, int size_z)
{
	// allocating memory for array of pointers to pointers
	T ***m=new T**[size_x];
	//	assert(m!=NULL);
	if (m == NULL) {
		throw error_msg("MEMORY_ERROR", "Memory can't be initialized: %s size", size_x * sizeof(T));
	}
	for (int x = 0; x < size_x; x++) {
		// for each pointer allocating memory for array of pointers
		m[x] = new T*[size_y];
		//		assert(m[x]!=NULL);
		if (m[x] == NULL) {
			throw error_msg("MEMORY_ERROR", "Memory can't be initialized: %s size", size_y * sizeof(T));
		}
	}
	//allocating memory for data array
	m[0][0] = new T[size_x * size_y * size_z]; // m[0][0] is pointer to array of T
	//	assert(m[0][0]!=NULL);
	if (m[0][0] == NULL) {
		throw error_msg("MEMORY_ERROR", "Memory can't be initialized: %s size", size_x * size_y * size_z * sizeof(T));
	}
	for (int x = 0; x < size_x; x++) {
		for (int y = 0; y < size_y; y++) {
			// assign pointers to data ranges
			m[x][y] = m[0][0] + (x*size_y + y)*size_z;
		}
	}

	return m;
}

/// Freeing memory for 1D matrix
/// \todo Move to Matrix.cpp
template<class T>inline void free_matrix(T* m) {
	delete m;
}

/// Freeing memory for 2D matrix
/// \todo Move to Matrix.cpp
template<class T>inline void free_matrix(T** m) {
	delete[](m[0]);
	delete[](m);
}

/// Freeing memory for 3D matrix
/// \todo Move to Matrix.cpp
template<class T>inline void free_matrix(T*** m, int size_x, int size_y) {
	delete[](m[0][0]);
	for (int x = 0; x < size_x; x++) {
		delete[](m[x]);
	}
	delete[](m);
}

//////////////////////////////////////////////////
//
//	Matrix 1D
//
//////////////////////////////////////////////////

/**
* Constructor.
*
* Runs allocating memory function and store matrix name.
*
* \param x_size - size of the matrix
* \param name - name of the matrix
*
*/
template<class T>
Matrix1D<T>::Matrix1D( int x_size , string name) {
	initialized = false;
	this->name = name;
	// allocating memory
	AllocateMemory(x_size);
}

/**
* Constructor.
*
* Runs allocating memory function.
*
* \param x_size - size of the matrix
*
*/
template<class T>
Matrix1D<T>::Matrix1D( int x_size ) {
	initialized = false;
	// allocating memory
	AllocateMemory(x_size);
}

/**
* Constructor.
*
* Make new matrix equal to Matrix M.
*
* \param &M - matrix M
*
*/
template<class T>
Matrix1D<T>::Matrix1D( const Matrix1D<T> &M ) {
	initialized = false;
	this->operator = (M);
}

/**
* Destructor. Destruct the class.
*/
template<class T>
Matrix1D<T>::~Matrix1D() {
	if (initialized) free_matrix<T>(matrix_array);
}

/**
* Allocating memory
*
* \param x_size - size x
*/
template<class T>
void Matrix1D<T>::AllocateMemory( int x_size ) {
	this->size_x = x_size;
	// using inline template for memory allocation
	matrix_array = matrix<T>(x_size);
	initialized = true;
	//	for (int i = 0; i < size_x; i++) matrix_array[i] = 2;
}


/**
* Operator [i], returns value of element i.
* If  DEBUG_MODE defined, check if matrix has been initialized.
*
* \param i - number of element to return
*/
template<class T>
inline T& Matrix1D<T>::operator[](int i) {
#ifdef DEBUG_MODE
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	}
#endif
	// return i-value
	return matrix_array[i];
}

/**
* Operator [i], returns value of element i, version returns 'const' value, can not be later modified.
* If  DEBUG_MODE defined, check if matrix has been initialized.
*
* \param i - number of element to return
*/
template<class T>
inline T& Matrix1D<T>::operator[](int i) const {
#ifdef DEBUG_MODE
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	}
#endif
	// return i-value
	return matrix_array[i];
}

/**
* Operator (x), returns value of element x.
* If  DEBUG_MODE defined, check if matrix has been initialized.
* No dofference between [] and () operators for 1d-matrix class.
*
* \param x - number of element to return
*/
template<class T>
inline T& Matrix1D<T>::operator()(int x) {
#ifdef DEBUG_MODE
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	}
#endif
	// return x-value
	return matrix_array[x];
}

/**
* Make matrix equal to matrix M
* Return the same instance of class Matrix.
*
* \param &M - matrix M
*/
template<class T>
inline Matrix1D<T>& Matrix1D<T>::operator= (const Matrix1D<T> &M) {
	// Check, if both matrix aren't the same one
	if (this!=&M) {
		// check, if RHS matrix was initialized
		if (M.initialized) {
			// free LHS matrix, if it was initialized
			if (initialized) free_matrix<T>(matrix_array);
			// copy parameters
			this->size_x = M.size_x;
			this->name = M.name;
			// allocating memory for LHS matrix
			this->AllocateMemory(this->size_x);
			// fast values copying as a memory range
			memcpy( matrix_array, M.matrix_array, this->size_x * sizeof( T ) );
		} else {
			this->initialized = false;
		}
	}
	return *this;
}

/**
* Make matrix equal to value Val.
* Return the same instance of class Matrix.
*
* \param Val - value val
*/
template<class T>
inline Matrix1D<T>& Matrix1D<T>::operator= (const T Val) {
	for (int x = 0; x < this->size_x; x++)
		matrix_array[x] = Val;
	return *this;
}

/**
* Multiply a matrix to a value Val.
* Return new instance of class Matrix.
*
* \param Val - value Val
*/
template<class T>
inline Matrix1D<T> Matrix1D<T>::operator* (const T Val) const {
	Matrix1D<T> Tmp(*this);
	for (int x = 0; x < this->size_x; x++)
		Tmp[x] = matrix_array[x] * Val;
	return Tmp;
}

/**
* Divide a matrix to a value Val.
* Return new instance of class Matrix.
*
* \param Val - value Val
*/
template<class T>
inline Matrix1D<T> Matrix1D<T>::operator/ (const T Val) const {
	int x;
	Matrix1D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		Tmp[x] = matrix_array[x] / Val;
	return Tmp;
}

/**
* Multiply all values of matrix to values of matrix M.
*
* \param &M - matrix M.
*/
template<class T>
inline Matrix1D<T> Matrix1D<T>::times (const Matrix1D<T> &M) const {
	int x;
	Matrix1D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		Tmp[x] = matrix_array[x] * M.matrix_array[x];
	return Tmp;
}

/**
* Divide all values of matrix to values of matrix M.
*
* \param &M - matrix M.
*/
template<class T>
inline Matrix1D<T> Matrix1D<T>::divide (const Matrix1D<T> &M) const {
	int x;
	Matrix1D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		Tmp[x] = matrix_array[x] / M.matrix_array[x];
	return Tmp;
}

/**
* Norm of vector
*/
template<class T>
inline T Matrix1D<T>::norm() const {
	T res = 0;
	int x;
	for (x = 0; x < this->size_x; x++) {
		res += matrix_array[x] * matrix_array[x];
	}
	return sqrt(res);
}

/**
* Dot product
*/
template<class T>
inline T Matrix1D<T>::dot( const Matrix1D<T> &W ) const {
	if (this->size_x != W.size_x)
		throw error_msg("DOT_PRODUCT", "Size is different");

	T res = 0;
	int x;
	for (x = 0; x < this->size_x; x++) {
		res += matrix_array[x] * W[x];
	}
	return res;
}


/**
* Write matrix data to file.
*/
template<class T>
void Matrix1D<T>::writeToFile(string filename) {
	int x;
	ofstream output(filename.c_str());
	if (output==NULL && (filename.find("Debug") == string::npos)) {
		throw error_msg("FILE", "Unable to output file: %s", filename.c_str());
	}
	output << "VARIABLES = \"" << ((this->name!="")?this->name:"function") << "\" "<< endl;
	output << "ZONE T=\"" << filename << "\", I=" << size_x << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (x = 0; x < size_x; x++) {
		output << matrix_array[x] << endl;
	}
	output.close();
}

/**
* Write matrix data to file with grid.
*/
template<class T>
void Matrix1D<T>::writeToFile(string filename, Matrix1D<T> &grid_x) {
	int x;
	ofstream output(filename.c_str());
	if (output==NULL && (filename.find("Debug") == string::npos)) {
		throw error_msg("FILE", "Unable to output file: %s", filename.c_str());
	}
	output << "VARIABLES = \"" << ((grid_x.name!="")?grid_x.name:"x") << "\", \""  << ((this->name!="")?this->name:"function") << "\" "<< endl;
	output << "ZONE T=\"" << filename << "\", I=" << size_x << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (x = 0; x < size_x; x++) {
		output << "\t" << grid_x[x] << "\t" << "\t" << matrix_array[x] << endl;
	}
	output.close();
}


/**
* Read matrix data from file.
*/
template<class T>
void Matrix1D<T>::readFromFile(string filename) {
	int x;
	string inBuf;
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	} else {
		ifstream input(filename.c_str());
		if (input != NULL && !input.eof()) {
			// Skipping first two lines.
			input >> inBuf;
			// !!!			while (strcmp(strupr((char *) inBuf.c_str()), "ZONE") != 0 && !input.eof() ) {
			while (strcasecmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {
				input >> inBuf;
			}
			// read to the end of the line with 'zone'
			input.ignore(9999, '\n');
			for (x = 0; x < size_x; x++) {
				input >> matrix_array[x];
			}
		} else {
			throw error_msg("MATRIX_LOAD_ERROR", "Error reading file %s.\n", filename.c_str());
			//cout << "Error reading initial conditions input file " << filename << endl;
			//getchar();
			//exit(-1);
		}
		input.close();
	}
}

/**
* Read matrix data from file and check 'grid'
*/
template<class T>
void Matrix1D<T>::readFromFile(string filename, Matrix1D<T> &grid_x) {
	int x;
	string inBuf;
	double loaded_x;
	double err = 1e-8;
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	} else {
		ifstream input(filename.c_str());
		if (input != NULL && !input.eof()) {
			// Skipping first two lines. Should serch for 'ZONE' and read from following line better - done.
			/// \todo All matrix readings change to readFromFile() function
			input >> inBuf;
			// !!!			while (strcmp(strupr((char *) inBuf.c_str()), "ZONE") != 0 && !input.eof() ) {
			while (strcasecmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {
				input >> inBuf;
			}
			// read to the end of the line with 'zone'
			input.ignore(9999, '\n');
			for (x = 0; x < size_x; x++) {
				input >> loaded_x;
				// check if grid is the same
				if (fabs(loaded_x - grid_x[x]) > err) {
					throw error_msg("MATRIX_LOAD_GRID_ERR", "Loading %s: grid mismatch.\nLoaded: %e\nGrid: %e\n", filename.c_str(), loaded_x, grid_x[x]);
				} else {
					input >> matrix_array[x];
				}
			}
		} else {
			throw error_msg("MATRIX_LOAD_ERROR", "Error reading file %s.\n", filename.c_str());
			//cout << "Error reading initial conditions input file " << filename << endl;
			//getchar();
			//exit(-1);
		}
		input.close();
	}
}


//////////////////////////////////////////////////
//
//	Matrix 2D
//
//////////////////////////////////////////////////

/**
* Constructor.
* Allocate memory.
*
* \param x_size - x size
* \param y_size - y size
*/
template<class T>
Matrix2D<T>::Matrix2D( int x_size, int y_size ) {
	initialized = false;
	// allocating memory
	AllocateMemory(x_size, y_size);
}

/**
* Constructor.
* Create new matrix from the Matrix M.
*
* \param &M - Matrix M
*/
template<class T>
Matrix2D<T>::Matrix2D( const Matrix2D<T> &M ) {
	initialized = false;
	this->operator = (M);
}

/**
* Destructor.
*/
template<class T>
Matrix2D<T>::~Matrix2D() {
	if (initialized) free_matrix<T>(matrix_array);
}

/**
* Allocate memory.
*
* \param x_size - x size
* \param y_size - y size
*/
template<class T>
void Matrix2D<T>::AllocateMemory( int x_size, int y_size ) {
	this->size_x = x_size;
	this->size_y = y_size;
	// using matrix inline template to allocate memory
	matrix_array = matrix<T>(x_size, y_size);
	initialized = true;
	//	for (int i = 0; i < size_x; i++)
	//		for (int j = 0; j < size_y; j++)
	//			matrix_array[i][j] = 0;
}

/**
* Make matrix equal to Matrix M.
*
* \param &M - Matrix M.
*/
template<class T>
Matrix2D<T>& Matrix2D<T>::operator= (const Matrix2D<T> &M) {
	// check, if both matrix are the same one
	if (this!=&M) {
		// check, if RHS matrix in initialized
		if (M.initialized) {
			// free LHS matrix if it was initialized
			if (initialized && (size_x != M.size_x || size_y != M.size_y)) {
				free_matrix<T>(matrix_array);
				initialized = false;
			}
			if (!initialized) {
				this->size_x = M.size_x;
				this->size_y = M.size_y;
				// allocating memory for LHS matrix (and creating the correct pointers M[0] - M[N] to the matrix rows)
				this->AllocateMemory(this->size_x, this->size_y);
			}
			this->name = M.name;
			//			for (int x = 0; x < this->size_x; x++)
			//				memcpy( matrix_array[x], M.matrix_array[x], this->size_y * sizeof( T ) );
			// fast values copying as a memory range
			memcpy(matrix_array[0], M.matrix_array[0], this->size_x * this->size_y * sizeof(T) );
		} else {
			this->initialized = false;
		}
	}
	return *this;
}

/**
* Make matrix equal to value Val.
*
* \param val - value Val.
*/
template<class T>
inline Matrix2D<T>& Matrix2D<T>::operator= (const T val) {
	int x, y;
	if (initialized) {
		for (x = 0; x < this->size_x; x++)
			for (y = 0; y < this->size_y; y++)
				matrix_array[x][y] = val;
	} else {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	}

	return *this;
}

/**
* Multiply matrix to Val.
*
* \param Val - value Val.
*/
template<class T>
inline Matrix2D<T> Matrix2D<T>::operator* (const T Val) const {
	int x, y;
	Matrix2D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			Tmp[x][y] = matrix_array[x][y] * Val;
	return Tmp;
}

/**
* Divide matrix to Val.
*
* \param Val - value Val.
*/
template<class T>
inline Matrix2D<T> Matrix2D<T>::operator/(const T Val) const {
	int x, y;
	Matrix2D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			Tmp[x][y] = matrix_array[x][y] / Val;
	return Tmp;
}

/**
* Divide all values of matrix to values of matrix M.
*
* \param &M - matrix M.
*/
template<class T>
inline Matrix2D<T> Matrix2D<T>::divide (const Matrix2D<T> &M) const {
	int x, y;
	Matrix2D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			Tmp[x][y] = matrix_array[x][y] / M[x][y];
	return Tmp;
}

/**
* Multiply all values of matrix to values of matrix M.
*
* \param &M - matrix M.
*/
template<class T>
inline Matrix2D<T> Matrix2D<T>::times (const Matrix2D<T> &M) const {
	int x, y;
	Matrix2D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			Tmp[x][y] = matrix_array[x][y] * M[x][y];
	return Tmp;
}

/**
* Divide all values of matrix to value Val.
*
* \param val - value to divide.
*/
template<class T>
Matrix2D<T> Matrix2D<T>::max_of(T val) {
	int x, y;
	Matrix2D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			Tmp[x][y] = (matrix_array[x][y]>val)?matrix_array[x][y]:val;
	return Tmp;
}

/**
* Returns corresponding index of 1d array
*/
template<class T>
inline int Matrix2D<T>::index1d(int x, int y) const {
	return x*size_y + y;
}



/**
* Writes the matrix to a file.
* File has two header lines.
*
* \todo Change the name 'writeToFile' in Matrix classes to 'write' or something.
*
* \param filename - file name
*/
template<class T>
void Matrix2D<T>::writeToFile(string filename) {
	int x, y;
	ofstream output(filename.c_str());
	if (output==NULL && (filename.find("Debug") == string::npos)) {
		throw error_msg("FILE", "Unable to output file: %s", filename.c_str());
	}
	output << "VARIABLES = \""<< ((this->name!="")?this->name:"f") << "\" "<< endl;
	output << "ZONE T=\"" << filename << "\", I=" << size_y << ", J= " << size_x << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			output << matrix_array[x][y] << endl;
		}
	}
	output.close();
}


/**
* Write the matrix to a file using other two matrixes as a grid.
* Simply that means - write all three matrixes to a file.
*/
template<class T>
void Matrix2D<T>::writeToFile(string filename, Matrix2D<T> &grid_x, Matrix2D<T> &grid_y) {
	int x, y;
	ofstream output(filename.c_str());
	output << "VARIABLES = \"" << ((grid_x.name!="")?grid_x.name:"x") << "\", \"" << ((grid_y.name!="")?grid_y.name:"y") << "\", \"" << ((this->name!="")?this->name:"f") << "\" "<< endl;
	output << "ZONE T=\"" << filename << "\", I=" << size_y << ", J=" << size_x << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			output << "\t" << grid_x[x][y] << "\t" << grid_y[x][y] << "\t" << matrix_array[x][y] << endl;
		}
	}
	output.close();
}


/**
* Read matrix data from file.
*/
template<class T>
void Matrix2D<T>::readFromFile(string filename) {
	int x, y;
	string inBuf;
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	} else {
		ifstream input(filename.c_str());
		if (input != NULL && !input.eof()) {
			// Skipping first two lines.
			input >> inBuf;
			// !!!			while (strcmp(strupr((char *) inBuf.c_str()), "ZONE") != 0 && !input.eof() ) {
			while (strcasecmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {
				input >> inBuf;
			}
			// read to the end of the line with 'zone'
			input.ignore(9999, '\n');
			for (x = 0; x < size_x; x++) {
				for (y = 0; y < size_y; y++) {
					input >> matrix_array[x][y];
				}
			}
		} else {
			throw error_msg("MATRIX_LOAD_ERROR", "Error reading file %s.\n", filename.c_str());
			//cout << "Error reading initial conditions input file " << filename << endl;
			//getchar();
			//exit(-1);
		}
		input.close();
	}
}

/**
* Read matrix data from file and check grid
*/
template<class T>
void Matrix2D<T>::readFromFile(string filename, Matrix2D<T> &grid_x, Matrix2D<T> &grid_y) {
	int x, y;
	string inBuf;
	double err = 1e-6;
	double loaded_x, loaded_y;
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	} else {
		ifstream input(filename.c_str());
		if (input != NULL && !input.eof()) {
			// Skipping first two lines.
			input >> inBuf;
			//	!!!		while (strcmp(strupr((char *) inBuf.c_str()), "ZONE") != 0 && !input.eof() ) {
			while (strcasecmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {
				input >> inBuf;
			}
			// read to the end of the line with 'zone'
			input.ignore(9999, '\n');
			for (x = 0; x < size_x; x++) {
				for (y = 0; y < size_y; y++) {
					input >> loaded_x >> loaded_y;
					// check if grid is the same
					if (fabs(loaded_x - grid_x[x][y]) > err || fabs(loaded_y - grid_y[x][y]) > err) {
						throw error_msg("MATRIX_LOAD_GRID_ERR", "Loading %s: grid mismatch.\nLoaded: %e, %e\nGrid: %e, %e\n", filename.c_str(), loaded_x, loaded_y, grid_x[x][y], grid_y[x][y]);
					} else {
						input >> matrix_array[x][y];
					}
				}
			}
		} else {
			throw error_msg("MATRIX_LOAD_ERROR", "Error reading file %s.\n", filename.c_str());
			//cout << "Error reading initial conditions input file " << filename << endl;
			//getchar();
			//exit(-1);
		}
		input.close();
	}
}


//////////////////////////////////////////////////
//
//	Matrix 3D
//
//////////////////////////////////////////////////

/**
* Constructor.
* Allocate memory.
*/
template<class T>
Matrix3D<T>::Matrix3D( int x_size, int y_size, int z_size ) {
	initialized = false;
	// allocating memory
	AllocateMemory(x_size, y_size, z_size);
}

/**
* Constructor.
* Create matrix equal to Matrix M.
*
* \param &M - Matrix M.
*/
template<class T>
Matrix3D<T>::Matrix3D( const Matrix3D<T> &M ) {
	initialized = false;
	this->operator = (M);
}

/**
* Destructor
*/
template<class T>
Matrix3D<T>::~Matrix3D() {
	if (initialized) free_matrix<T>(matrix_array, size_x, size_y);
	//		delete[] plane_array; - it's get deleted with matrix_array[0][0][0] or something?
}

/**
* Allocating memory and filling it with zero-values.
*/
template<class T>
void Matrix3D<T>::AllocateMemory( int x_size, int y_size, int z_size) {
	this->size_x = x_size;
	this->size_y = y_size;
	this->size_z = z_size;
	matrix_array = matrix<T>(x_size, y_size, z_size);
	plane_array = matrix_array[0][0];
	initialized = true;
#ifdef DEBUG_MODE
	// should not initialize matrix with zeros, it can slow the code greatly in some cases
	for (int i = 0; i < size_x; i++)
		for (int j = 0; j < size_y; j++)
			for (int k = 0; k < size_z; k++)
				matrix_array[i][j][k] = 0;
#endif
}

/**
* Operator [i], returns pointer to 2D array. Next [j][k] can be applied to return value.
* If  DEBUG_MODE defined, check if matrix has been initialized.
*
* \param i - number of element to return
*/
template<class T>
inline T** Matrix3D<T>::operator[] (int i) {
#ifdef DEBUG_MODE
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	}
#endif
	// return i-th pointer
	return matrix_array[i];
}

/**
* Operator (x, y, z), returns value of element [x][y][z].
* If  DEBUG_MODE defined, check if matrix has been initialized.
*
*/
template<class T>
inline T& Matrix3D<T>::operator() (int x, int y, int z) {
#ifdef DEBUG_MODE
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using not initialized matrix");
	}
	if ((x < 0 || x > size_x-1) || (y < 0 || y > size_y-1) || (z < 0 || z > size_z-1)) {
		throw error_msg("MATRIX_ERROR", "Index is out of bound");
	}
#endif
	// return (x,y,z) value
	return plane_array[(x*size_y + y)*size_z + z];
}

/**
* Makes matrix equal to Matrix M.
*
* \param &M - Matrix M.
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator= (const Matrix3D<T> &M) {
	// check if not LHS and RHS matrix are the same
	if (this!=&M) {
		// check if RHS matrix is initialized
		if (M.initialized) {
			// free LHS matrix, if it is initialized
			if (initialized && (size_x != M.size_x || size_y != M.size_y || size_z != M.size_z)) {
				free_matrix<T>(matrix_array, this->size_x, this->size_y);
				initialized = false;
			}
			if (!initialized) {
				this->size_x = M.size_x;
				this->size_y = M.size_y;
				this->size_z = M.size_z;
				// allocating memory for LHS matrix
				this->AllocateMemory(this->size_x, this->size_y, this->size_z);
			}
			this->name = M.name;
			//			for (int x = 0; x < this->size_x; x++)
			//				for (int y = 0; y < this->size_y; y++)
			//					memcpy( matrix_array[x][y], M.matrix_array[x][y], this->size_z * sizeof( T ) );
			// fast values copying as memory range
			memcpy( matrix_array[0][0], M.matrix_array[0][0], this->size_x * this->size_y * this->size_z * sizeof( T ) );
		} else {
			this->initialized = false;
		}
	}
	return *this;
}

/**
* Makes 3D matrix from 2D matrix.
* The 3rd dimension makes equal to 1.
*/
/* template<class T>
Matrix3D<T>& Matrix3D<T>::operator= (const Matrix2D<T> &M) {
double val;
if (M.initialized) {
if (initialized) free_matrix<T>(matrix_array, this->size_x, this->size_y);
this->size_x = 1;
this->size_y = M.size_x;
this->size_z = M.size_y;
//this->name = M.name;
this->AllocateMemory(this->size_x, this->size_y, this->size_z);
for (int y = 0; y < this->size_y; y++)
//memcpy( matrix_array[0][y], M.matrix_array[y], this->size_z * sizeof( T ) );
for (int z = 0; z < this->size_z; z++) {
this->matrix_array[0][y][z] = M[y][z];
}
} else {
this->initialized = false;
}
return *this;
}*/

/**
* Makes Matrix equal to value Val.
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator= (const T Val) {
	// std::fill_n(matrix_array[0][0][0], size_x*size*y*size*z, Val); // might be faster
	for (int x = 0; x < this->size_x; x++)
		for (int y = 0; y < this->size_y; y++)
			for (int z = 0; z < this->size_z; z++)
				this->matrix_array[x][y][z] = Val;
	return *this;
}


/**
* Matrix summation, result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator+= (const Matrix3D<T> &M) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] += M.matrix_array[x][y][z];
	return *this;
}

/**
* Matrix subtraction, result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator-= (const Matrix3D<T> &M) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] -= M.matrix_array[x][y][z];
	return *this;
}

/**
* Multiplication to a value. Result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator*= (const T Val) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] *= Val;
	return *this;
}

/**
* Division by a value. Result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator/= (const T Val) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] /= Val;
	return *this;
}

/**
* Summation with a value. Result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator+= (const T Val) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] += Val;
	return *this;
}

/**
* Subtraction of a value. Result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::operator-= (const T Val) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] -= Val;
	return *this;
}

/**
* Multiplication between each element of the matrices (not a matrix multiplication). Result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::times_equal (const Matrix3D<T> &M) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] *= M.matrix_array[x][y][z];
	return *this;
}

/**
* Division of each element of one matrices to the element of another. Result is stored into applied matrix (left hand side matrix)
*/
template<class T>
inline Matrix3D<T>& Matrix3D<T>::divide_equal (const Matrix3D<T> &M) {
	int x, y, z;
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				matrix_array[x][y][z] /= M.matrix_array[x][y][z];
	return *this;
}

/**
* Add each element of the matrix to corresponds element of matrix M.
*/
template<class T>
inline Matrix3D<T> Matrix3D<T>::operator+ (const Matrix3D<T> &M) const {
	int x, y, z;
	Matrix3D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				Tmp[x][y][z] = matrix_array[x][y][z] + M.matrix_array[x][y][z];
	return Tmp;
}

/**
* Substract each element of the matrix to corresponds element of matrix M.
*/
template<class T>
inline Matrix3D<T> Matrix3D<T>::operator- (const Matrix3D<T> &M) const {
	int x, y, z;
	Matrix3D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				Tmp[x][y][z] = matrix_array[x][y][z] - M.matrix_array[x][y][z];
	return Tmp;
}


/**
* Multiply each element of the matrix to Val, save result to a new matrix.
*/
template<class T>
inline Matrix3D<T> Matrix3D<T>::operator* (const T Val) const {
	int x, y, z;
	Matrix3D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				Tmp[x][y][z] = matrix_array[x][y][z] * Val;
	return Tmp;
}


/**
* Divide each element of the matrix to Val, save result to a new matrix.
*/
template<class T>
inline Matrix3D<T> Matrix3D<T>::operator/ (const T Val) const {
	int x, y, z;
	Matrix3D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				Tmp[x][y][z] = matrix_array[x][y][z] / Val;
	return Tmp;
}

/**
* Multiply each element of the matrix to corresponds element of matrix M.
*/
template<class T>
inline Matrix3D<T> Matrix3D<T>::times (const Matrix3D<T> &M) const {
	int x, y, z;
	Matrix3D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				Tmp[x][y][z] = matrix_array[x][y][z] * M.matrix_array[x][y][z];
	return Tmp;
}

/**
* Divide each element of the matrix to corresponds element of matrix M.
*/
template<class T>
inline Matrix3D<T> Matrix3D<T>::divide (const Matrix3D<T> &M) const {
	int x, y, z;
	Matrix3D<T> Tmp(*this);
	for (x = 0; x < this->size_x; x++)
		for (y = 0; y < this->size_y; y++)
			for (z = 0; z < this->size_z; z++)
				Tmp[x][y][z] = matrix_array[x][y][z] / M.matrix_array[x][y][z];
	return Tmp;
}


/**
* Write matrix to file.
* File has two header lines.
*/
template<class T>
void Matrix3D<T>::writeToFile(string filename) {
	int x, y, z;
	ofstream output(filename.c_str());
	output << "VARIABLES = \""<< ((this->name!="")?this->name:"f") <<"\" "<< endl;
	output << "ZONE T=\"" << filename << "\", I=" << size_z << ", J=" << size_y << ", K=" << size_x << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			for (z = 0; z < size_z; z++) {
				output << matrix_array[x][y][z] << endl;
			}
		}
	}
	output.close();
}

/**
* Write matrix to file, using 3 other matrixes as a grid (simply - write all 4 matrixes to the file).
* File has two header lines.
*/
template<class T>
void Matrix3D<T>::writeToFile(string filename, Matrix3D<T> &grid_x, Matrix3D<T> &grid_y, Matrix3D<T> &grid_z) {
	int x, y, z;
	ofstream output(filename.c_str());
	if (output==NULL && (filename.find("Debug") == string::npos)) {
		throw error_msg("FILE", "Unable to output file: %s", filename.c_str());
	}
	output << "VARIABLES = \"" << ((grid_x.name!="")?grid_x.name:"x") << "\", \"" << ((grid_y.name!="")?grid_y.name:"y") << "\", \"" << ((grid_z.name!="")?grid_z.name:"z") << "\", \"" << ((this->name!="")?this->name:"f") << "\" "<< endl;
	output << "ZONE T=\"" << filename << "\", I=" << size_z << ", J=" << size_y << ", K=" << size_x << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			for (z = 0; z < size_z; z++) {
				output << "\t" << grid_x[x][y][z] << "\t" << grid_y[x][y][z] << "\t" << grid_z[x][y][z] << "\t" << matrix_array[x][y][z] << endl;
			}
		}
	}
	output.close();
}

/**
* Read matrix data from file.
*/
template<class T>
void Matrix3D<T>::readFromFile(string filename) {
	int x, y, z;
	string inBuf;
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	} else {
		ifstream input(filename.c_str());
		if (input != NULL && !input.eof()) {
			// Skipping first two lines.
			input >> inBuf;
			// !!!			while (strcmp(strupr((char *) inBuf.c_str()), "ZONE") != 0 && !input.eof() ) {
			while (strcasecmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {

				input >> inBuf;
			}
			// read to the end of the line with 'zone'
			input.ignore(9999, '\n');
			for (x = 0; x < size_x; x++) {
				for (y = 0; y < size_y; y++) {
					for (z = 0; z < size_z; z++) {
						input >> matrix_array[x][y][z];
					}
				}
			}
		} else {
			throw error_msg("MATRIX_LOAD_ERROR", "Error reading file %s.\n", filename.c_str());
			//cout << "Error reading initial conditions input file " << filename << endl;
			//getchar();
			//exit(-1);
		}
		input.close();
	}
}

/**
* Read matrix data from file with grid
*/
template<class T>
void Matrix3D<T>::readFromFile(string filename, Matrix3D<T> &grid_x, Matrix3D<T> &grid_y, Matrix3D<T> &grid_z) {
	int x, y, z;
	string inBuf;
	if (!initialized) {
		throw error_msg("MATRIX_ERROR", "Using unitialized matrix");
	} else {
		ifstream input(filename.c_str());
		if (input != NULL && !input.eof()) {
			// Skipping first two lines.
			input >> inBuf;
			// !!!			while (strcmp(_strupr((char *) inBuf.c_str()), "ZONE") != 0 && !input.eof() ) {
			// !!!			while (strcasecmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {
			while (strcmp(inBuf.c_str(), "ZONE") != 0 && !input.eof() ) {
				input >> inBuf;
			}
			// read to the end of the line with 'zone'
			input.ignore(9999, '\n');
			for (x = 0; x < size_x; x++) {
				for (y = 0; y < size_y; y++) {
					for (z = 0; z < size_z; z++) {
						input >> grid_x[x][y][z] >> grid_y[x][y][z] >> grid_z[x][y][z] >> matrix_array[x][y][z];
						// skip till the end of the line
						input.ignore(9999, '\n');

						// input >> loaded_x >> loaded_y >> loaded_z;
						// check if grid is the same
						// if (fabs(loaded_x - grid_x[x][y][z]) > err || fabs(loaded_y - grid_y[x][y][z]) > err || fabs(loaded_z - grid_z[x][y][z]) > err) {
						//	throw error_msg("MATRIX_LOAD_GRID_ERR", "Loading %s: grid mismatch.\nLoaded: %e, %e, %e\nGrid: %e, %e, %e\n", filename.c_str(), loaded_x, loaded_y, loaded_z, grid_x[x][y][z], grid_y[x][y][z], grid_z[x][y][z]);
						//} else {
						//	input >> matrix_array[x][y][z];
						//}
					}
				}
			}
		} else {
			throw error_msg("MATRIX_LOAD_ERROR", "Error reading file %s.\n", filename.c_str());
			//cout << "Error reading initial conditions input file " << filename << endl;
			//getchar();
			//exit(-1);
		}
		input.close();
	}
}



/**
* Returns corresponding index of 1d array
*/
template<class T>
inline int Matrix3D<T>::index1d(int x, int y, int z) {
	return (x*size_y + y)*size_z + z;
}


/**
* Return maximum value of the matrix.
*/
template<class T>
T Matrix3D<T>::max() {
	T tmp = 0;
	int x, y, z;
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			for (z = 0; z < size_z; z++) {
				tmp = (tmp>matrix_array[x][y][z])?tmp:matrix_array[x][y][z];
			}
		}
	}
	return tmp;
}

/**
* Return absolute maximum value of the matrix.
*/
template<class T>
T Matrix3D<T>::maxabs() {
	T tmp = 0;
	int x, y, z;
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			for (z = 0; z < size_z; z++) {
				tmp = (tmp>fabs((double)matrix_array[x][y][z]))?tmp:fabs((double)matrix_array[x][y][z]);
			}
		}
	}
	return tmp;
}

/**
* Return absolute value of the matrix.
*/
template<class T>
Matrix3D<T> Matrix3D<T>::abs() {
	Matrix3D<T> tmp(this->size_x, this->size_y, this->size_z);
	int x, y, z;
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			for (z = 0; z < size_z; z++) {
				tmp[x][y][z] = (matrix_array[x][y][z]>0)?matrix_array[x][y][z]:-matrix_array[x][y][z];
			}
		}
	}
	return tmp;
}


/**
* Make x-slice of 3d matrix - 2d matrix.
*/
template<class T>
Matrix2D<T> Matrix3D<T>::xSlice(int p_x) const {
	int y, z;
	Matrix2D<T> tmp(this->size_y, this->size_z);
	tmp.name = this->name + "_slice";
	for (y = 0; y < size_y; y++) {
		for (z = 0; z < size_z; z++) {
			tmp[y][z] = matrix_array[p_x][y][z];
		}
	}
	return tmp;
}

/**
* Make y-slice of 3d matrix - 2d matrix.
*/
template<class T>
Matrix2D<T> Matrix3D<T>::ySlice(int p_y) const {
	int x, z;
	Matrix2D<T> tmp(this->size_x, this->size_z);
	tmp.name = this->name + "_slice";
	for (x = 0; x < size_x; x++) {
		for (z = 0; z < size_z; z++) {
			tmp[x][z] = matrix_array[x][p_y][z];
		}
	}
	return tmp;
}

/**
* Make z-slice of 3d matrix - 2d matrix.
*/
template<class T>
Matrix2D<T> Matrix3D<T>::zSlice(int p_z) const {
	int x, y;
	Matrix2D<T> tmp(this->size_x, this->size_y);
	tmp.name = this->name + "_slice";
	for (x = 0; x < size_x; x++) {
		for (y = 0; y < size_y; y++) {
			tmp[x][y] = matrix_array[x][y][p_z];
		}
	}
	return tmp;
}


/////////////////////////////////////////////////////////////////////////////
//
//		Interpolations (mess)
//
//////////////////////////////////////////////////////////////////////////////


/**
* Linear interpolation.
* Runs interpolation function from Numerical Recepies.
*/
template<class T>
void Matrix1D<T>::Interpolate(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid) {


	//if (size_x != old_grid.size_x) {
	if (this->size_x != new_grid.size_x) {
		throw error_msg("INTERPOLATION_ERROR", "New grid and function size mismatch!\n");
		//cout << "new grid and function size mismatch!" << endl;
		//return;
	}

	//Matrix1D<T> res(new_grid.size_x);
	//Maths::Interpolation::Linear LinInterp(old_grid.size_x, old_grid.matrix_array, matrix_array);
	// initialize some class. It was like that when I downloaded the function.
	Maths::Interpolation::Linear LinInterp(old_grid.size_x, old_grid.matrix_array, old_function.matrix_array);

	int i;
	for (i = 0; i < new_grid.size_x; i++)
		//res[i] = LinInterp.getValue(new_grid.matrix_array[i]);
		matrix_array[i] = LinInterp.getValue(new_grid.matrix_array[i]);

	//return res;

}

/**
* Spline interpolation.
* Runs interpolation functions from Numerical Recipies.
* Treats boundary conditions separate, i.e. use boundary values on boundary conditions and extrapolation, use linear interpolation for next-to-the-boundary points.
* Has additional mechanism of smoothening the interpolation - makes it more linear according to the parameters lin_spline_coef and max_second_der.
* Spline interpolation is linear + additional terms from second derivatives (look Numerical Recipies). If second derivative > max_second_der the second derivatives are multiplied by lin_spline_coef.
*
* \param &old_function - old function
* \param &old_grid - old grid
* \param &new_grid - new grid
* \param lb - low boundary value
* \param ub - upper boundary value
* \param lin_spline_coef - coefficient to multiply the second derivatives terms (makes interpolation more smooth)
* \param max_second_der - maximum second derivatives, all greater derivatives gonna be multiplied by lin_spline_coef
*
*/
template<class T>
void Matrix1D<T>::Spline(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid, double lb, double ub, double lin_spline_coef, double max_second_der) {

	//if (size_x != old_grid.size_x) {
	// check, if size of the grids are equal
	if (this->size_x != new_grid.size_x) {
		throw error_msg("SPLINE_INTERPOLATION", "New grid and function size mismatch!");
	}
	// if size == 1 - just copy the value
	if (this->size_x == 1) {
		this->matrix_array[0] = old_function.matrix_array[0];
		return;
	}

	//  Idea was to find the intersection between grids, to interpolate insite the intersection and do not extrapolate outside. But is done later.
	int i=0, i_min=0, i_max=old_grid.size_x-1, interp_size=0;
	//	for(i = 0; old_grid.matrix_array[i] <= new_grid.matrix_array[0] && i < old_grid.size_x-1; i++) {}
	//	i_min = i;
	//	for(i = old_grid.size_x-1; old_grid.matrix_array[i] >= new_grid.matrix_array[old_grid.size_x-1] && i > 0; i--) {}
	//	i_max = i;
	//cout << i_min << " " << i_max << endl;

	// new range for spline interpolation - values close to the boundaries are interpolated by linear interpolation
	i_min = 1; i_max = old_grid.size_x-2;
	interp_size = i_max-i_min+1;

	double yp0, ypn;
	//yp0 = 2*(old_function[1] -  old_function[0])/(old_grid[1] - old_grid[0]) - (old_function[2] -  old_function[1])/(old_grid[2] - old_grid[1]);
	//ypn = 2*(old_function[size_x-1] -  old_function[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]) - (old_function[size_x-2] -  old_function[size_x-3])/(old_grid[size_x-2] - old_grid[size_x-3]);
	//	ypn = (old_function[size_x-1] -  old_function[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]);

	// flag, means use smooth second derivative on the boundary
	yp0 = 1e99;
	//yp0 = -(old_function[2] -  old_function[0])/(old_grid[2] - old_grid[0]);
	ypn = 1e99;//0;

	Matrix1D<T> y2(old_grid.size_x);
	//spline(old_grid.matrix_array, old_function.matrix_array, old_grid.size_x, yp0, ypn, y2.matrix_array);
	//spline(&old_grid.matrix_array[1], &old_function.matrix_array[1], old_grid.size_x-2, yp0, ypn, &y2.matrix_array[1]);
	// calculating array of second derivatives
	spline(&old_grid.matrix_array[i_min], &old_function.matrix_array[i_min], interp_size, yp0, ypn, &y2.matrix_array[i_min]);

	// make it more linear (lower second derivatives)
	for (i = 0; i < old_grid.size_x; i++) {
		// if second derivative value is too big
		if (fabs(y2[i]) >= max_second_der) {
			if (y2[i] > 0) {
				// decreasing by lin_spline_coef big second derivative values - makes interpolation more linear
				// see numerical recipes
				y2[i] = max_second_der + (y2[i] - max_second_der)*(1.0 - lin_spline_coef);
			} else {
				y2[i] = - max_second_der + (y2[i] + max_second_der)*(1.0 - lin_spline_coef);
			}
		}
	}

	// for all grid points - looking for new values
	double a; int k; // interpolation parameters
	for (i = 0; i < new_grid.size_x; i++) {
		if (i == 0) { // if we are on the boundary
			this->matrix_array[i] = lb;
		} else if (i == new_grid.size_x-1) { // if we are on the boundary
			this->matrix_array[i] = ub;
		} else 	if (new_grid.matrix_array[i] <= old_grid.matrix_array[0] ) { // if it is extrapolation
			this->matrix_array[i] = lb;
			//this->matrix_array[i] = old_function.matrix_array[0];
		} else if (new_grid.matrix_array[i] >= old_grid.matrix_array[old_grid.size_x-1]) { // if it is extrapolation
			this->matrix_array[i] = ub;
			//this->matrix_array[i] = old_function.matrix_array[old_function.size_x-1];
		} else if (new_grid.matrix_array[i] <= old_grid.matrix_array[1] ) {  // if the new grid point is between two boundary points of the old grid
			// linear
			double a = (new_grid.matrix_array[i] - old_grid.matrix_array[0])/(old_grid.matrix_array[1] - old_grid.matrix_array[0]);
			this->matrix_array[i] = old_function.matrix_array[0] + a*(old_function.matrix_array[1] - old_function.matrix_array[0]);
		} else if (new_grid.matrix_array[i] >= old_grid.matrix_array[old_grid.size_x-2]) {  // if the new grid point is between two boundary points of the old grid
			// linear
			//double a = (new_grid.matrix_array[i] - old_grid.matrix_array[old_grid.size_x-2])/(old_grid.matrix_array[old_grid.size_x-1] - old_grid.matrix_array[old_grid.size_x-2]);
			//this->matrix_array[i] = old_function.matrix_array[old_grid.size_x-2] + a*(old_function.matrix_array[old_grid.size_x-1] - old_function.matrix_array[old_grid.size_x-2]);
			double a = (new_grid.matrix_array[i] - old_grid.matrix_array[old_grid.size_x-1])/(old_grid.matrix_array[old_grid.size_x-2] - old_grid.matrix_array[old_grid.size_x-1]);
			this->matrix_array[i] = old_function.matrix_array[old_grid.size_x-1] + a*(old_function.matrix_array[old_grid.size_x-2] - old_function.matrix_array[old_grid.size_x-1]);
			/*		} else if (i <= 1) { // !!!!!!!! always linear for the first two points ???
			int k = 1;
			while(old_grid.matrix_array[k] < new_grid.matrix_array[i]) {k++;}
			double a = (new_grid.matrix_array[i] - old_grid.matrix_array[k-1])/(old_grid.matrix_array[k] - old_grid.matrix_array[k-1]);
			this->matrix_array[i] = old_function.matrix_array[k-1] + a*(old_function.matrix_array[k] - old_function.matrix_array[k-1]);
			} else if (i >= new_grid.size_x-2) { // !!!!!!!! always linear for the last two points ???
			int k = new_grid.size_x-1;
			while(old_grid.matrix_array[k] > new_grid.matrix_array[i]) {k--;}
			double a = (new_grid.matrix_array[i] - old_grid.matrix_array[k])/(old_grid.matrix_array[k+1] - old_grid.matrix_array[k]);
			this->matrix_array[i] = old_function.matrix_array[k] + a*(old_function.matrix_array[k+1] - old_function.matrix_array[k]);*/
		} else {
			//splint(old_grid.matrix_array, old_function.matrix_array, y2.matrix_array, old_grid.size_x, new_grid.matrix_array[i], &this->matrix_array[i]);
			//splint(&old_grid.matrix_array[1], &old_function.matrix_array[1], &y2.matrix_array[1], old_grid.size_x-2, new_grid.matrix_array[i], &this->matrix_array[i]);
			// Shouldn't we pass whole matrix for the least argument????
			/// \todo Check the index of the last argument of the spline interpolation.
			splint(&old_grid.matrix_array[i_min], &old_function.matrix_array[i_min], &y2.matrix_array[i_min], interp_size, new_grid.matrix_array[i], &this->matrix_array[i]);
		}
	}

}

/*template<class T>
void Matrix1D<T>::Spline(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid, double ub, double lb) {

//if (size_x != old_grid.size_x) {
if (this->size_x != new_grid.size_x) {
cout << "new grid and function size mismatch!" << endl;
return;
}

double yp0, ypn;
//yp0 = 2*(old_function[1] -  old_function[0])/(old_grid[1] - old_grid[0]) - (old_function[2] -  old_function[1])/(old_grid[2] - old_grid[1]);
//ypn = 2*(old_function[size_x-1] -  old_function[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]) - (old_function[size_x-2] -  old_function[size_x-3])/(old_grid[size_x-2] - old_grid[size_x-3]);
//	ypn = (old_function[size_x-1] -  old_function[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]);

yp0 = 1e99;//-(old_function[1] -  old_function[0])/(old_grid[1] - old_grid[0]);
ypn = 1e99;//0;

Matrix1D<double> y2(old_grid.size_x);
//spline(old_grid.matrix_array, old_function.matrix_array, old_grid.size_x, yp0, ypn, y2.matrix_array);
spline(&old_grid.matrix_array[1], &old_function.matrix_array[1], old_grid.size_x-2, yp0, ypn, &y2.matrix_array[1]);

int i;
for (i = 0; i < new_grid.size_x; i++) {
if (i == 0) {
this->matrix_array[i] = lb;
} else if (i == new_grid.size_x-1) {
this->matrix_array[i] = ub;
} else
if (new_grid.matrix_array[i] <= old_grid.matrix_array[0] ) {
this->matrix_array[i] = lb;//old_function.matrix_array[0];
} else if (new_grid.matrix_array[i] <= old_grid.matrix_array[1] ) {
// linear
double a = (new_grid.matrix_array[i] - old_grid.matrix_array[0])/(old_grid.matrix_array[1] - old_grid.matrix_array[0]);
this->matrix_array[i] = old_function.matrix_array[0] + a*(old_function.matrix_array[1] - old_function.matrix_array[0]);
} else if (new_grid.matrix_array[i] >= old_grid.matrix_array[old_grid.size_x-1]) {
this->matrix_array[i] = ub;//1.e-21; //old_function.matrix_array[old_function.size_x-1];
} else if (new_grid.matrix_array[i] >= old_grid.matrix_array[old_grid.size_x-2]) {
// linear
//double a = (new_grid.matrix_array[i] - old_grid.matrix_array[old_grid.size_x-2])/(old_grid.matrix_array[old_grid.size_x-1] - old_grid.matrix_array[old_grid.size_x-2]);
//this->matrix_array[i] = old_function.matrix_array[old_grid.size_x-2] + a*(old_function.matrix_array[old_grid.size_x-1] - old_function.matrix_array[old_grid.size_x-2]);
double a = (new_grid.matrix_array[i] - old_grid.matrix_array[old_grid.size_x-1])/(old_grid.matrix_array[old_grid.size_x-2] - old_grid.matrix_array[old_grid.size_x-1]);
this->matrix_array[i] = old_function.matrix_array[old_grid.size_x-1] + a*(old_function.matrix_array[old_grid.size_x-2] - old_function.matrix_array[old_grid.size_x-1]);
} else {
//splint(old_grid.matrix_array, old_function.matrix_array, y2.matrix_array, old_grid.size_x, new_grid.matrix_array[i], &this->matrix_array[i]);
splint(&old_grid.matrix_array[1], &old_function.matrix_array[1], &y2.matrix_array[1], old_grid.size_x-2, new_grid.matrix_array[i], &this->matrix_array[i]);
}
}

}*/

/**
* Another spline interpolation. An attempt to make it more stable.
* Runs interpolation functions from Numerical Recepies.
* Treats boundary conditions separete, i.e. use boundary values on baundary conditions and extrapolation, use linear interpolation for next-to-the-boundary points.
* Has additional mechenism of smoothning the interpolation - makes it more linear according the parameters lin_spline_coef and max_second_der.
* Spline interpolation is linear + additional terms from second derivatives (look Numerical Recepies). If second derivative > max_second_der the second derivatives are multiplid by lin_spline_coef.
*
* \param &old_function - old function
* \param &old_grid - old grid
* \param &new_grid - new grid
* \param lb - low boundary value
* \param ub - upper boundary value
* \param lin_spline_coef - coefficient to multiply the second derivatives terms (makes interpolation more smooth)
* \param max_second_der - maximum second derivatives, all greater derivatives gonna be multiplied by lin_spline_coef
*
*/
template<class T>
void Matrix1D<T>::Spline2(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid, double lb, double ub, double lin_spline_coef, double max_second_der) {

	//if (size_x != old_grid.size_x) {
	// check, if size of the grids are equal
	if (this->size_x != new_grid.size_x) {
		throw error_msg("SPLINE_INTERPOLATION", "New grid and function size mismatch!");
	}
	// if size == 1 - just copy the value
	if (this->size_x == 1) {
		this->matrix_array[0] = old_function.matrix_array[0];
		return;
	}

	//  Idea was to find the intersection between grids, to interpolate inside the intersection and do not extrapolate outside. But is done later.
	int i=0, i_min=0, i_max=old_grid.size_x-1, interp_size=0;
	//	for(i = 0; old_grid.matrix_array[i] <= new_grid.matrix_array[0] && i < old_grid.size_x-1; i++) {}
	//	i_min = i;
	//	for(i = old_grid.size_x-1; old_grid.matrix_array[i] >= new_grid.matrix_array[old_grid.size_x-1] && i > 0; i--) {}
	//	i_max = i;
	//cout << i_min << " " << i_max << endl;

	// new range for spline interpolation - values close to the boundaries are interpolated by linear interpolation
	i_min = 0; i_max = old_grid.size_x-1;
	interp_size = i_max-i_min+1;

	double yp0, ypn;
	//yp0 = 2*(old_function[1] -  old_function[0])/(old_grid[1] - old_grid[0]) - (old_function[2] -  old_function[1])/(old_grid[2] - old_grid[1]);
	//ypn = 2*(old_function[size_x-1] -  old_function[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]) - (old_function[size_x-2] -  old_function[size_x-3])/(old_grid[size_x-2] - old_grid[size_x-3]);
	//	ypn = (old_function[size_x-1] -  old_function[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]);

	// flag, means use smooth second derivative on the boundary
	yp0 = 1e99;
	//yp0 = -(old_function[2] -  old_function[0])/(old_grid[2] - old_grid[0]);
	ypn = 1e99;//0;

	Matrix1D<T> y2(old_grid.size_x);
	// calculating array of second derivatives
	spline(&old_grid.matrix_array[i_min], &old_function.matrix_array[i_min], interp_size, yp0, ypn, &y2.matrix_array[i_min]);

	// for all grid points - looking for new values
	for (i = 0; i < new_grid.size_x; i++) {
		splint(&old_grid.matrix_array[i_min], &old_function.matrix_array[i_min], &y2.matrix_array[i_min], interp_size, new_grid.matrix_array[i], &this->matrix_array[i]);
	}

}


/**
* polynomial + linear
* Some other weird interpolation attempts.
*/
template<class T>
void Matrix1D<T>::Polilinear(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid, double lb, double ub) {

	//if (size_x != old_grid.size_x) {
	if (this->size_x != new_grid.size_x) {
		throw error_msg("POLINOMIAL_INTERPOLATION", "New grid and function size mismatch!");
	}

	int i;
	for (i = 0; i < new_grid.size_x; i++)
		matrix_array[i] = polilinear( old_grid.matrix_array, old_function.matrix_array, old_grid.size_x, new_grid.matrix_array[i], lb, ub);

}

/**
* polynomial
* Some other weird interpolation attempts.
*/
template<class T>
void Matrix1D<T>::Polint(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid) {

	//if (size_x != old_grid.size_x) {
	if (this->size_x != new_grid.size_x) {
		throw error_msg("POLINT_INTERPOLATION", "New grid and function size mismatch!");
	}

	int i;
	double err;
	for (i = 0; i < new_grid.size_x; i++)
		polint( old_grid.matrix_array, old_function.matrix_array, old_grid.size_x, new_grid.matrix_array[i], &this->matrix_array[i], &err);

}

/// rational
template<class T>
void Matrix1D<T>::Ratint(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid) {

	//if (size_x != old_grid.size_x) {
	if (this->size_x != new_grid.size_x) {
		throw error_msg("RATIONAL_INTERPOLATION", "New grid and function size mismatch!");
	}

	int i;
	double err;
	for (i = 0; i < new_grid.size_x; i++)
		ratint( old_grid.matrix_array, old_function.matrix_array, old_grid.size_x, new_grid.matrix_array[i], &this->matrix_array[i], &err);

}

//template<class T>
//Matrix1D<T> Matrix1D<T>::Spline(Matrix1D<T> &old_grid, Matrix1D<T> &new_grid) {
//
//	if (this->size_x != old_grid.size_x) {
//	//if (this->size_x != new_grid.size_x) {
//		cout << "old grid and function size mismatch!" << endl;
//		return -1;
//	}
//
//	double yp0, ypn;
//	yp0 = (matrix_array[0] -  matrix_array[1])/(old_grid[0] - old_grid[1]);
//	ypn = (matrix_array[size_x-1] -  matrix_array[size_x-2])/(old_grid[size_x-1] - old_grid[size_x-2]);
//	Matrix1D<T> y2(old_grid.size_x);
//	spline(old_grid.matrix_array, this->matrix_array, old_grid.size_x, yp0, ypn, y2.matrix_array)
//
//	int i;
//	Matrix1D<T> res(new_grid.size_x);
//	for (i = 0; i < new_grid.size_x; i++)
//		spline(old_grid.matrix_array, this->matrix_array, y2.matrix_array, new_grid.matrix_array[i], &res[i]);
//
//	return res;
//
//}


// interpolations

/**
* Akima interpolation
*/
template<class T>
void Matrix1D<T>::Akima_interpolation(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid, int extrapolation_type, double lb, double ub) {

	/*
	for (i = 0; i < new_grid.size_x; i++) {
	if (i == 0) { // if we are on the boundary
	this->matrix_array[i] = lb;
	} else if (i == new_grid.size_x-1) { // if we are on the boundary
	this->matrix_array[i] = ub;
	} else 	if (new_grid.matrix_array[i] <= old_grid.matrix_array[0] ) { // if it is extrapolation
	this->matrix_array[i] = lb;
	//this->matrix_array[i] = old_function.matrix_array[0];
	} else if (new_grid.matrix_array[i] >= old_grid.matrix_array[old_grid.size_x-1]) { // if it is extrapolation
	this->matrix_array[i] = ub;
	//this->matrix_array[i] = old_function.matrix_array[old_function.size_x-1];
	} else if (new_grid.matrix_array[i] <= old_grid.matrix_array[1] ) {  // if the new grid point is between two boundaryes poins of the old grid
	// linear
	double a = (new_grid.matrix_array[i] - old_grid.matrix_array[0])/(old_grid.matrix_array[1] - old_grid.matrix_array[0]);
	this->matrix_array[i] = old_function.matrix_array[0] + a*(old_function.matrix_array[1] - old_function.matrix_array[0]);
	} else if (new_grid.matrix_array[i] >= old_grid.matrix_array[old_grid.size_x-2]) {  // if the new grid point is between two boundaryes poins of the old grid
	// linear
	//double a = (new_grid.matrix_array[i] - old_grid.matrix_array[old_grid.size_x-2])/(old_grid.matrix_array[old_grid.size_x-1] - old_grid.matrix_array[old_grid.size_x-2]);
	//this->matrix_array[i] = old_function.matrix_array[old_grid.size_x-2] + a*(old_function.matrix_array[old_grid.size_x-1] - old_function.matrix_array[old_grid.size_x-2]);
	double a = (new_grid.matrix_array[i] - old_grid.matrix_array[old_grid.size_x-1])/(old_grid.matrix_array[old_grid.size_x-2] - old_grid.matrix_array[old_grid.size_x-1]);
	this->matrix_array[i] = old_function.matrix_array[old_grid.size_x-1] + a*(old_function.matrix_array[old_grid.size_x-2] - old_function.matrix_array[old_grid.size_x-1]);
	} else if (i <= 1) { // always linear for the first point?
	int k = 1;
	//			Wrong! We need to compare to grids, not indexes. Changed in ver 1.04.
	//			while(k < i) {k++;}
	while(old_grid.matrix_array[k] < new_grid.matrix_array[i]) {k++;}
	double a = (new_grid.matrix_array[i] - old_grid.matrix_array[k-1])/(old_grid.matrix_array[k] - old_grid.matrix_array[k-1]);
	this->matrix_array[i] = old_function.matrix_array[k-1] + a*(old_function.matrix_array[k] - old_function.matrix_array[k-1]);
	} else {
	akima(old_grid.matrix_array, old_function.matrix_array, old_function.size_x, new_grid.matrix_array, this->matrix_array, this->size_x, ub, lb);
	}
	}*/


	// add point
	/*int i_first = 0, i_last = new_grid.size_x-1;
	while(old_grid.matrix_array[i_first] <= new_grid.matrix_array[0]) {i_first++;}
	while(old_grid.matrix_array[i_last] >= new_grid.matrix_array[new_grid.size_x-1]) {i_last--;}
	Matrix1D<double> mod_grid(i_last-i_first+2);
	mod_grid[0] = lb;
	mod_grid[mod_grid.x_size-1] = ub;

	for (int i = 1; i < mod_grid.size_x-1; i++) {
	mod_grid[i] = old_grid.matrix_array[i_first + i - 1];
	}

	akima(mod_grid.matrix_array, old_function.matrix_array, old_function.size_x, new_grid.matrix_array, this->matrix_array, this->size_x, 1, ub, lb);*/


	/*int i_first = 0, i_last = new_grid.size_x-1;
	while(new_grid.matrix_array[i_first] <= old_grid.matrix_array[0]) {i_first++;}
	while(new_grid.matrix_array[i_last] >= old_grid.matrix_array[old_grid.size_x-1]) {i_last--;}
	int new_size_x = i_last - i_first + 1;

	akima(old_grid.matrix_array, old_function.matrix_array, old_function.size_x,
	&new_grid.matrix_array[i_first], &this->matrix_array[i_first], new_size_x,
	2,
	ub, lb);

	// boundaries (and extrapolation)
	int i;
	for (i = 0; i < i_first; i++) this->matrix_array[i] = lb;
	for (i = this->size_x-1; i > i_last; i--) this->matrix_array[i] = ub;*/

	// new grid is exactly one point wider than old grid
	/*int old_first = 0, old_last = old_grid.size_x-1;
	while(old_grid.matrix_array[old_first] <= new_grid.matrix_array[0]) {old_first++;}
	while(old_grid.matrix_array[old_last] >= new_grid.matrix_array[new_grid.size_x-4]) {old_last--;}
	int old_size_x = old_last - old_first + 1;
	int new_first = 0, new_last = new_grid.size_x-1;
	while(new_grid.matrix_array[new_first+1] <= old_grid.matrix_array[0]) {new_first++;}
	while(new_grid.matrix_array[new_last-1] >= old_grid.matrix_array[old_grid.size_x-4]) {new_last--;}
	int new_size_x = new_last - new_first + 1;

	akima(&old_grid.matrix_array[old_first], &old_function.matrix_array[old_first], old_size_x,
	&new_grid.matrix_array[new_first], &this->matrix_array[new_first], new_size_x, 1, ub, lb);

	// boundaries (and extrapolation)
	int i;
	for (i = 0; i < new_first; i++) this->matrix_array[i] = lb;
	for (i = this->size_x-1; i > new_last; i--) this->matrix_array[i] = ub;*/

	///
	/*		int i_first = 0, i_last = new_grid.size_x-1;
	while(old_grid.matrix_array[i_first] <= new_grid.matrix_array[0]) {i_first++;}
	while(old_grid.matrix_array[i_last] >= new_grid.matrix_array[new_grid.size_x-1]) {i_last--;}
	int new_size_x = i_last - i_first + 1;

	//akima(&old_grid.matrix_array[i_first], &old_function.matrix_array[i_first], new_size_x, new_grid.matrix_array, this->matrix_array, this->size_x, 1, ub, lb);
	//		akima(old_grid.matrix_array, old_function.matrix_array, old_function.size_x, new_grid.matrix_array, this->matrix_array, this->size_x, 1, ub, lb);
	akima(old_grid.matrix_array, old_function.matrix_array, old_function.size_x,
	&new_grid.matrix_array[i_first], &this->matrix_array[i_first], new_size_x,
	2,
	ub, lb,
	max(this->matrix_array[0], old_grid.matrix_array[0]), min(this->matrix_array[this->x_size-1], old_grid.matrix_array[old_grid.x_size-1]));

	// boundaries (and extrapolation)
	int i;
	for (i = 0; i < i_first; i++) this->matrix_array[i] = lb;
	for (i = this->size_x-1; i > i_last; i--) this->matrix_array[i] = ub;*/



	akima(old_grid.matrix_array, old_function.matrix_array, old_function.size_x, new_grid.matrix_array, this->matrix_array, this->size_x, 1, ub, lb);
	matrix_array[0] = lb;
	matrix_array[size_x-1] = ub;
}


/**
* Akima interpolation - second test version
*/
template<class T>
void Matrix1D<T>::Akima_interpolation2(Matrix1D<T> &old_function, Matrix1D<T> &old_grid, Matrix1D<T> &new_grid, int extrapolation_type, double lb, double ub) {
	akima2(old_grid.matrix_array, old_function.matrix_array, old_function.size_x, new_grid.matrix_array, this->matrix_array, this->size_x, ub, lb);
}

/**
* Linear2D. Does not work probably or works only for regular grid.
* Some other wired interpolation attempts.
*/
template<class T>
double Linear2D(double x, double y, T &old_grid_x, T &old_grid_y, T &f) {

	int i = 0, j = 0;

	while (x >= old_grid_x[++i][0] && i < old_grid_y.size_y-1);
	while (y >= old_grid_y[0][++j] && j < old_grid_x.size_x-1);

	double t = (x - old_grid_x[i-1][0])/(old_grid_x[i][0] - old_grid_x[i-1][0]);
	double u = (y - old_grid_y[0][j-1])/(old_grid_y[0][j] - old_grid_y[0][j-1]);

	return (1.0-t)*(1.0-u)*f[i-1][j-1] + t*(1.0-u)*f[i][j-1] + t*u*f[i][j] + (1.0-t)*u*f[i-1][j];
}


/**
* Linear 2D interpolation.
*
* Not sure if it is working.
*/
template<class T>
void Matrix2D<T>::Interpolate(Matrix2D<T> &old_function, Matrix2D<T> &old_grid_x, Matrix2D<T> &old_grid_y, Matrix2D<T> &new_grid_x, Matrix2D<T> &new_grid_y) {

	Matrix2D<T> res(new_grid_x.size_x, new_grid_y.size_y);

	if (old_function.size_x != old_grid_x.size_x || old_function.size_y != old_grid_y.size_y) {
		throw error_msg("LINEAR_INTERPOLATION", "Old grid and function size mismatch!");
	}

	if (this->size_x != new_grid_x.size_x || this->size_y != new_grid_y.size_y) {
		throw error_msg("LINEAR_INTERPOLATION", "New grid and function size mismatch!");
	}

	int i, j;
	for (i = 0; i < new_grid_x.size_x; i++) {
		for (j = 0; j < new_grid_y.size_y; j++) {
			this->matrix_array[i][j] = Linear2D(new_grid_x[i][j], new_grid_y[i][j], old_grid_x, old_grid_y, old_function);
		}
	}

	return;

}


/**
* Constructor for CalculationMatrix class.
* \param x_size - x size
* \param y_size - y size
* \param z_size - z size
* \param n_of_diags - NUMBER OF DIAGONALS ABOVE THE MAIN DIAGONAL (main diagonal is not counted)
*/
CalculationMatrix::CalculationMatrix(int x_size, int y_size, int z_size, int n_of_diags) {
	this->initialized = false;
	Initialize(x_size, y_size, z_size, n_of_diags);
}

/**
* Allocating memory for CalculationMatrix
*/
void CalculationMatrix::Initialize(int x_size, int y_size, int z_size, int n_of_diags) {
	this->initialized = false;
	this->x_size = x_size;
	this->y_size = y_size;
	this->total_size = x_size;
	if (y_size > 0) this->total_size = this->total_size * y_size;
	if (z_size > 0) this->total_size = this->total_size * z_size;


	// initializing diagonals
	int x, y, z, id;
	// !!!	if (z_size > 0) { // means 3d
	if (z_size > 1) { // means 3d
		for (x = -n_of_diags; x <= n_of_diags; x++) {
			for (y = -n_of_diags; y <= n_of_diags; y++) {
				for (z = -n_of_diags; z <= n_of_diags; z++) {
					// calculating a diagonal number (id)
					id = this->index1d(x, y, z);
					// allocating memory for each diagonal
					(*this)[id] = Matrix1D<double>(this->total_size);
					(*this)[id] = 0;
				}
			}
		}
		// !!!	} else if (y_size > 0) {
	} else if (y_size > 1) {
		for (x = -n_of_diags; x <= n_of_diags; x++) {
			for (y = -n_of_diags; y <= n_of_diags; y++) {
				// calculating a diagonal number (id)
				id = this->index1d(x, y);
				// allocating memory for each diagonal
				(*this)[id] = Matrix1D<double>(this->total_size);
				(*this)[id] = 0;
			}
		}
		// !!!	} else if (x_size > 0) {
	} else if (x_size > 1) {
		for (x = -n_of_diags; x <= n_of_diags; x++) {
			// calculating a diagonal number (id)
			id = this->index1d(x);
			// allocating memory for each diagonal
			(*this)[id] = Matrix1D<double>(this->total_size);
			(*this)[id] = 0;
		}
	}

	// If im_size is not set, then it's 1d and ia_size should be zero for future
	if (y_size == 0) this->x_size = 0;
	// If il_size is not set, then it's 2d and im_size should be zero for future
	if (z_size == 0) this->y_size = 0;

	this->initialized = true;
}

/**
* Function returns 1d index for 2D-3D arrays
*/
int CalculationMatrix::index1d(int x, int y, int z) {
	return (z*y_size + y)*x_size + x;
}

/**
* Save matrix to file.
*/
void CalculationMatrix::writeToFile(string filename) {
	int in;

	ofstream output(filename.c_str());
	if (output==NULL && (filename.find("Debug") == string::npos)) {
		throw error_msg("FILE", "Unable to output file: %s", filename.c_str());
	}
	output << "VARIABLES = ";
	for (DiagMatrix::iterator it = (*this).begin(); it != (*this).end(); it++) {
		output << "\"" << it->first << "\", ";
	}
	output << endl;
	output << "ZONE T=\"" << "\", I=" << total_size << endl;
	output.setf(ios_base::scientific, ios_base::floatfield);
	for (in = 0; in < this->total_size; in++) {
		for (DiagMatrix::iterator it = (*this).begin(); it != (*this).end(); it++) {
			output << "\t" << it->second[in];
		}
		output << endl;
	}
	output.close();
}


//////////////////////////////////////////
// Implementations
//////////////////////////////////////////

template class Matrix1D<double>;
template class Matrix2D<double>;
template class Matrix3D<double>;


#endif
