/**
 * \file variousConstants.h 
 * \brief Various useful constants, see source code for details
 * \author Developed under supervision of the PI Yuri Shprits
 */

#ifndef VARIOUS_CONSTANTS_H

/// \def VARIOUS_CONSTANTS_H
/// \brief Prevent of double definition of this file
#define VARIOUS_CONSTANTS_H

/** \namespace VC
 * \brief Various mathematical constants Click variousConstants.h to see constants in source code.
 *
 *  Namespace defined in file. It has no cpp files because in contains only constants.
 */

namespace VC {

	/// \brief Minimum value for PSD
	static const double zero_f = 1e-31; 
	///\brief m*c^2 = 0.511 MeV
	static const double mc2	   = 0.511;
	///\brief &pi;
	static const double pi	   = 3.141592653589793238462643;
	///\brief mass of electron in grams
	static const double m	   = 9.10953e-28;
	///\brief speed of light, cm/s
	static const double c_cgs  = 2.998e10;
	///\brief speed of light, km/s
	static const double c_si   = 2.998e8;
	///\brief electron charge, CGS units
	static const double qe	   = 4.803242e-10;
	///\brief magnetic field at the equatorial plane on the Earth surface, Gauss
	static const double B_0	   = 0.311;
	///\brief exponential
	static const double exp	   = 2.7183;
	///\brief Radius fo the Earch in km
	static const double RE	   = 6.371e6;
}

#endif
