addpath('./Various_scripts')

clear all;
%close all

plot_sign_PSD = false

save_movie = false;
save_pictures = true;
movie_name = '../Plots/movie1';
picture_name = '../Plots/PSD_time';


path_file_arr = [
    {'../Output/'}
    ];

c_min = -8;
c_max = -4;

target_L = 2.4;
DOY_start=0;%293;


for i = 1:length(path_file_arr)
    path_file = path_file_arr{i}

    %gridFileName = [path_file, 'perp_grid.plt'];
    %dataFileName = [path_file, 'OutPSD.dat'];
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    gridFileName = [path_file, 'rad_grid.plt'];
    dataFileName = [path_file, 'OutPSD_rad.dat'];

    oneDimDataFileName = [path_file, 'out1d.dat'];

    [L, epc, alpha, pc] = load_plt(gridFileName, 'squeeze', 'permute', 'make3D');
    [time, Kp, Bf, Lpp] = load_plt(oneDimDataFileName);
    
    % compressions: Indeo5
    if (save_movie) mov = avifile([path_file, movie_name, '.avi'], 'fps', 10, 'quality', 0, 'compression', 'none'); end;
    
    if (plot_sign_PSD)
        f1 = figure('Units', 'inches', 'Position', [0.5 0.5 12 5], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 12, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 12,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%
    else
        f1 = figure('Units', 'inches', 'Position', [0.5 0.5 7 5], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 12, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 12,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%
    end
    
    % to read PSD only once, and then save it as .mat file
    try
        a(0); 
        load([dataFileName, '.mat'])
    catch
        PSD = load_plt(dataFileName, 'permute', 'nosqueeze', 'n_zones', 9999, 'skip_zones', 0);%
        save([dataFileName, '.mat'], 'PSD')
    end
    units_constant = 1/(3e7); % (c/MeV/cm)^3 after that
    PSD.arr = PSD.arr*units_constant;
    
    PSD.time = PSD.time + DOY_start;
    
    [val, L_slice] = min(abs(L.arr(:,1,1) - target_L));
    
    alpha_2d = squeeze(alpha.arr(L_slice,:,:))./pi.*180;
    energy_2d = squeeze(epc.arr(L_slice,:,:));
    
    set(gcf, 'Name', path_file);
    
    % comment/uncomment the loop if you need just one pitcture
    time_slice = length(PSD.time);
    for time_slice = 1:1:length(PSD.time)
        PSD.time(time_slice)
        
        if (plot_sign_PSD)
            subplot(1, 2, 1);
        end
        
        set(gca,'nextplot','replacechildren');
        
        PSD_2d = squeeze(PSD.arr(time_slice,L_slice,:,:));
        log_abs_PSD = log10(abs(PSD_2d));
        log_abs_PSD(isinf(log_abs_PSD)) = 0;
        log_abs_PSD(find(log_abs_PSD>c_max)) = c_max;
        log_abs_PSD(find(log_abs_PSD<c_min)) = c_min;
        
        %contourf(alpha_2d, energy_2d, log_abs_PSD, 50);
        pcolor(alpha_2d, energy_2d, log_abs_PSD);
        set(gca, 'yscale', 'log');
        shading flat
        
        caxis([c_min c_max]);
        
        ax_pos = get(gca, 'Position');
        cl = colorbar('West');
        cl_pos = get(cl, 'Position');
        cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;
        set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
        
        text(max(xlim), 10^((min(log10(ylim)) + max(log10(ylim)))/2), 'log_{10}(|PSD|, 3 \cdot 10^{-8} c/MeV/cm^3)', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold');
        text(min(xlim), max(ylim), ['Time=', num2str(PSD.time(time_slice)), ', days'], 'VerticalAlignment', 'top', 'HorizontalAlignment', 'left', 'FontWeight', 'bold', 'Color', [0.5, 0.5, 0.5]);
        
        xlabel('Pitch angle, deg')
        ylabel('Energy, MeV')
        
        % if we want to plot sing(PSD)
        if (plot_sign_PSD)
            subplot(1, 2, 2);
            set(gca,'nextplot','replacechildren');
            
            %contourf(alpha_2d, energy_2d, sign(PSD_2d));
            pcolor(alpha_2d, energy_2d, sign(PSD_2d));
            shading flat
            set(gca, 'yScale', 'log')
            caxis([-1 1])
            
            ax_pos = get(gca, 'Position');
            cl = colorbar('West');
            cl_pos = get(cl, 'Position');
            cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;%0.015; %0.925
            set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
            
            xlabel('Pitch angle, deg');
            ylabel('Energy, MeV');
            text(max(xlim), 10^((min(log10(ylim)) + max(log10(ylim)))/2), 'sign(PSD)', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold');
        end
        
        if (save_movie) mov = addframe(mov, getframe(f1)); end;
        getframe(f1);
        if (save_pictures) save_picture(gcf, [path_file, picture_name, '=', num2str(PSD.time(time_slice))], 'jpg', 'pdf', 'eps'); end;
    end
    
    if (save_movie) mov = close(mov); end;
    
end

