clear all;
addpath('./Various_scripts')

%close all

path_file_arr = [
    {'../Execute/Output/'}
];

target_epc = 1;
target_alpha = 80;

for pic = 1:length(path_file_arr)
    path_file = path_file_arr{pic};

    %gridFileName = [path_file, 'perp_grid.plt'];
    %dataFileName = [path_file, 'OutPSD.dat'];
    dataFileName = [path_file, 'OutPSD_rad.dat'];
    gridFileName = [path_file, 'rad_grid.plt'];

    [L, epc, alpha, pc] = load_plt(gridFileName, 'squeeze', 'permute');

    target_L = L.arr(:,1,1);
    [new_L, new_epc, new_alpha] = meshgrid(target_L, target_epc, target_alpha/180*pi);
    new_pc = pfunc(new_epc);
    new_epc = Kfunc(new_pc);

    % load PSD
    try
        %a(0)
        %delete([dataFileName, '.mat'])
        load([dataFileName, '.mat'])
    catch
        PSD = load_plt(dataFileName, 'permute', 'n_zones', 999, 'skip_zones', 6);%
        save([dataFileName, '.mat'], 'PSD')
    end
    units_constant = 1/3e7; % (c/MeV/cm)^3 after that
    PSD.arr = PSD.arr*units_constant;

    oneDimDataFileName = [path_file, 'out1d.dat'];
    [time, Kp, Bf, Lpp, Bw2] = load_plt(oneDimDataFileName);

    % interpolate PSD at specified energy and pitch-angle
    PSD_interpolated = fast_interpolate(L.arr, epc.arr, alpha.arr, PSD.arr, new_L, new_epc, new_alpha, 'log');
    
    % convert into fluxes
    for it = 1:size(PSD.arr,1)
        new_Flux(it, :) = PSD_interpolated(it, :) .* new_pc .* new_pc;
    end
    
    c_max = log10(max(new_Flux(:)));
    c_min = c_max - 3; % number of orders of magnitude to plot

    new_Flux(find(new_Flux < 10^c_min)) = 10^c_min;
    log_Flux = log10(new_Flux);
    
    %--------------------------------------------------------------------------
    % Plot 
    %--------------------------------------------------------------------------

    f1 = figure('Units', 'inches', 'Position', [1. 1. 8.5 4.5], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%

    cmap = colormap;
    cmap(1,:);
    cmap(1,:) = [0 0 0];
    colormap(cmap)

    subplot(4, 1, 1);
    plot(time.arr, Kp.arr, 'k');
    ylabel('Kp');

    title({path_file, ''});
    
    subplot(4, 1, 2);
    plot(time.arr, Bf.arr, 'k');
    ylabel('Bf');
    set(gca, 'yscale', 'log')

    subplot(2,1,2);
    contourf(PSD.time, new_L, log_Flux', 100);
    shading flat
    axis tight
    ylim([1 7]);
    caxis([c_min c_max]);
    
    % colorbar
    ax_pos = get(gca, 'Position');
    cl = colorbar('West');
    cl_pos = get(cl, 'Position');
    cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;%0.015; %0.925
    set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
    
    text(max(xlim), (min(ylim) + max(ylim))/2, ' Flux, log(#/sr/s/cm^2/KeV)', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold');

    hold on
    plot(time.arr, Lpp.arr, '--', 'color', 'white', 'Linewidth', 1);
    %plot(time.arr, Kp.arr, ':', 'color', 'white', 'Linewidth', 1);
    
    xlabel('Time, days');
    ylabel('L*');
    
    if (target_alpha >= 88)
        text(max(xlim), min(ylim), {['Energy = ', num2str(target_epc), ' MeV, equatorial pitch angle ']}, 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'color', 'white');
    else
        text(max(xlim), min(ylim), {['Energy = ', num2str(target_epc), ' MeV, Pitch angle = ', num2str(target_alpha), '^o ']}, 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'color', 'white');
    end

    set(gcf, 'Name', path_file);
    
    save_picture(gcf, ['Flux_E=', num2str(target_epc), '_alpha=', num2str(target_alpha), '_', num2str(c_max)], 'jpg', 'pdf', 'epsc', 'fig');
    
end
