clear all;
addpath('./Various_scripts')

%close all

path_file_arr = [
    {'../Execute/Output/'}
    ];

%target_mu = 700.0;
target_mu = [700];
%target_mu = [2000];

%target_Jc = [0.01];
%target_Jc = [1];
%target_K = 0.11;
target_K = [0.11];

mc0 = 0.511;%/10^8/10^8;
speed_of_light = 3e8;
earth_rad = 6.4e6;
mc2 = 0.511;
%target_K = target_Jc./sqrt(8.*mc0.*target_mu);
%target_Jc = target_K*sqrt(8*mc0*target_mu);

%-------------------------------------------------------------------------

for pic = 1:length(path_file_arr)
    path_file = path_file_arr{pic}

    %gridFileName = [path_file, 'perp_grid.plt'];
    %dataFileName = [path_file, 'OutPSD.dat'];
    dataFileName = [path_file, 'OutPSD_rad.dat'];
    gridFileName = [path_file, 'rad_grid.plt'];
    
    % Load grid
    [L, epc, alpha, pc] = load_plt(gridFileName, 'squeeze', 'permute');

    target_Jc = K2Jc(target_K, target_mu);
    new_L = L.arr(:,1,1);
    new_alpha = LmuJc2alpha(new_L, target_mu, target_Jc);
    new_pc = Lmualpha2pc(new_L', target_mu, new_alpha);
    new_epc = Kfunc(new_pc);
    
    % load PSD
    try
        %a(0)
        %delete([dataFileName, '.mat'])
        load([dataFileName, '.mat'])
    catch
        PSD = load_plt(dataFileName, 'permute', 'n_zones', 999, 'skip_zones', 6);%
        save([dataFileName, '.mat'], 'PSD')
    end
    units_constant = 1/3e7; % (c/MeV/cm)^3 after that
    PSD.arr = PSD.arr*units_constant;
    
    oneDimDataFileName = [path_file_arr{1}, 'out1d.dat'];
    [time, Kp, Bf, Lpp] = load_plt(oneDimDataFileName);

    % interpolate PSD at specified mu and J
    [new_PSD] = fast_interpolate(L.arr, epc.arr, alpha.arr, PSD.arr, new_L, new_epc, new_alpha, 'log');
    
    c_max = log10(max(new_PSD(:)));
    c_min = c_max - 3; % number of orders of magnitude to plot
    
    new_PSD(find(new_PSD < 10^c_min)) = 10^c_min;
    log_PSD = log10(new_PSD);
    
    %--------------------------------------------------------------------------
    % Plot 
    %--------------------------------------------------------------------------
    
    f1 = figure('Units', 'inches', 'Position', [1. 1. 8.5 4.5], ...
        'DefaultAxesFontName', 'times new roman', 'DefaultAxesFontWeight', 'bold', 'DefaultAxesFontSize', 10, 'DefaultTextFontWeight', 'bold', 'DefaultTextFontSize', 10,...
        'PaperOrientation', 'portrait', 'PaperPositionMode', 'auto', 'DefaultLineLineWidth', 2);%, 'PaperPosition', [1 1 3.8 9]);%
    
    cmap = colormap;
    cmap(1,:);
    cmap(1,:) = [0 0 0];
    colormap(cmap)
        
    subplot(4, 1, 1);
    plot(time.arr, Kp.arr, 'k');
    ylabel('Kp');

    title({path_file, ''});
    
    subplot(4, 1, 2);
    plot(time.arr, Bf.arr, 'k');
    ylabel('Bf');
    set(gca, 'yscale', 'log')
    
    subplot(2,1,2);    
    contourf(PSD.time, new_L, log_PSD', 100);
    shading flat
    axis tight
    ylim([1 7]);
    caxis([c_min c_max]);

    % colorbar    
    ax_pos = get(gca, 'Position');
    cl = colorbar('West');
    cl_pos = get(cl, 'Position');
    cl_x_pos = ax_pos(1) + ax_pos(3) + 0.025;%0.015; %0.925
    set(cl, 'Position', [cl_x_pos, cl_pos(2), cl_pos(3), cl_pos(4)]);
    
    text(max(xlim), (min(ylim) + max(ylim))/2, ' PSD, (c/cm/MeV)^3', 'Rotation', 90, 'VerticalAlignment', 'top', 'HorizontalAlignment', 'center', 'FontWeight', 'bold', 'FontSize', 10);

    
    hold on
    plot(time.arr, Lpp.arr, '--', 'color', 'white', 'Linewidth', 1);
    %plot(time.arr, Kp.arr, ':', 'color', 'white', 'Linewidth', 1);
    
    xlabel('Time, days');
    ylabel('L*');

    text(max(xlim), min(ylim), {['\mu = ', num2str(target_mu), ' MeV/G, K = ', num2str(target_K), ' G^{0.5} R_E ']}, 'VerticalAlignment', 'bottom', 'HorizontalAlignment', 'right', 'FontWeight', 'bold', 'color', 'white');
    
    set(gcf, 'Name', path_file);

    save_picture(gcf, ['PSD_mu=', num2str(target_mu), '_J=', num2str(target_Jc), '_K=', num2str(target_K)], 'jpg', 'pdf', 'epsc', 'fig');
    
end
